/***************************************************************
 * Name:      ApameGUIMain.h
 * Purpose:   Defines Application Frame
 * Author:    Daniel Filkovic (daniel.filkovic@gmail.com)
 * Created:   2010-10-23
 * Copyright: Daniel Filkovic (www.3dpanelmethod.com)
 * License:   GNU GENERAL PUBLIC LICENSE
 **************************************************************/

#ifndef APAMEGUIMAIN_H
#define APAMEGUIMAIN_H

//(*Headers(ApameGUIFrame)
#include <wx/menu.h>
#include <wx/glcanvas.h>
#include <wx/toolbar.h>
#include <wx/filedlg.h>
#include <wx/frame.h>
#include <wx/statusbr.h>
//*)
#include <wx/aboutdlg.h>
#include <wx/event.h>
#include <wx/timer.h>

#include "C3DViewer.h"
#include "CGUIData.h"
#include "CMesh.h"
#include "CProjectData.h"
#include "CResults.h"
#include "DAnalysisReport.h"
#include "DChangeElemType.h"
#include "DConfirmation.h"
#include "DCreateElement.h"
#include "DCreateNode.h"
#include "DDeleteElements.h"
#include "DDeleteNodes.h"
#include "DDisplayResults.h"
#include "DEditElement.h"
#include "DEditNode.h"
#include "DEditWakes.h"
#include "DElemSelection.h"
#include "DExportAbaqus.h"
#include "DExportNastran.h"
#include "DExportVTK.h"
#include "DExtrudeLine.h"
#include "DFlowParams.h"
#include "DGenerateNeighbors.h"
#include "DGenerateWake.h"
#include "DImportAbaqus.h"
#include "DImportApameSolverInput.h"
#include "DImportApameSolverResults.h"
#include "DImportFluent.h"
#include "DImportIdeas.h"
#include "DImportNastran.h"
#include "DImportVTK.h"
#include "DMergeNodes.h"
#include "DNodeSelection.h"
#include "DReferenceValues.h"
#include "DResultsRequest.h"
#include "DRun.h"
#include "DSeparateNeighbors.h"
#include "DSettings.h"
#include "DSolverParams.h"
#include "DTransformMirror.h"
#include "DTransformRotate.h"
#include "DTransformScale.h"
#include "DTransformTranslate.h"
#include "typenames.h"
#include "undoRedo.h"

class ApameGUIFrame: public wxFrame
{
public:
    ApameGUIFrame(
            wxWindow* parent,
            string& programArgument,
            wxWindowID id=-1);
    virtual ~ApameGUIFrame();
    void initialize();

private:
    // following data pointers contain all the data within ApameGUI project
    CGUIData*     m_pGUIData;     // GUI data
    CMesh*        m_pMesh;        // mesh data
    CProjectData* m_pProjectData; // project data
    CResults*     m_pResults;     // results data
    // declare undo/redo lists
    list<undoRedoStruct> m_undoList;
    list<undoRedoStruct> m_redoList;
    // declare node/element selection maps
    setUInt m_nodeSelection;
    setUInt m_elemSelection;
    // bool to flag if project has been modified
    bool m_modified;

    void refreshGUI();
    void processChange(bool modified, undoRedoStruct& undoInfo);
    void completeRefresh();

    //(*Handlers(ApameGUIFrame)
    void OnNew(wxCommandEvent& event);
    void OnMenuItemOpenSelected(wxCommandEvent& event);
    void OnMenuItemSaveSelected(wxCommandEvent& event);
    void OnMenuItemSaveAsSelected(wxCommandEvent& event);
    void OnClose(wxCloseEvent& event);
    void OnQuitSelected(wxCommandEvent& event);
    void OnMenuItemImportApameSolverInputSelected(wxCommandEvent& event);
    void OnMenuItemImportApameSolverResultsSelected(wxCommandEvent& event);
    void OnMenuItemImportAbaqusInputSelected(wxCommandEvent& event);
    void OnMenuItemImportFluentMeshSelected(wxCommandEvent& event);
    void OnMenuIdeas(wxCommandEvent& event);
    void OnMenuNastran(wxCommandEvent& event);
    void OnMenuItemImportVTKMeshSelected(wxCommandEvent& event);
    void OnMenuItemExportApameInputSelected(wxCommandEvent& event);
    void OnMenuItemExportAbaqusSelected(wxCommandEvent& event);
    void OnMenuItemExportNastranSelected(wxCommandEvent& event);
    void OnMenuItemExportVTK(wxCommandEvent& event);
    void OnMenuItemUndoSelected(wxCommandEvent& event);
    void OnMenuItemRedoSelected(wxCommandEvent& event);
    void OnMenuItemSelectNodesSelected(wxCommandEvent& event);
    void OnMenuItemSelectElemsSelected(wxCommandEvent& event);
    void OnMenuItemScaleSelected(wxCommandEvent& event);
    void OnMenuItemTranslateSelected(wxCommandEvent& event);
    void OnMenuItemRotateSelected(wxCommandEvent& event);
    void OnMenuItemMirrorSelected(wxCommandEvent& event);
    void OnMenuItemChangeElemTypeSelected(wxCommandEvent& event);
    void OnMenuItemEditWakesSelected(wxCommandEvent& event);
    void OnMenuItemNodeSelected(wxCommandEvent& event);
    void OnMenuItemElemSelected(wxCommandEvent& event);
    void OnMenuItemNeighborsSelected(wxCommandEvent& event);
    void OnMenuItemWakeSelected(wxCommandEvent& event);
    void OnMenuItemExtrudeLineSelected(wxCommandEvent& event);
    void OnMenuItemDSeparateNeighborsSelected(wxCommandEvent& event);
    void OnMenuItemMergeNodesSelected(wxCommandEvent& event);
    void OnMenuItemNodeDistSelected(wxCommandEvent& event);
    void OnMenuItemElemDistSelected(wxCommandEvent& event);
    void OnMenuItemCalcSurfSelected(wxCommandEvent& event);
    void OnMenuItemBoundBoxSelected(wxCommandEvent& event);
    void OnMenuItemFlowParams(wxCommandEvent& event);
    void OnMenuRefs(wxCommandEvent& event);
    void OnMenuSolvParams(wxCommandEvent& event);
    void OnMenuResultsRequest(wxCommandEvent& event);
    void OnMenuRunSelected(wxCommandEvent& event);
    void OnMenuItemWireframeSelected(wxCommandEvent& event);
    void OnMenuItemSolidSelected(wxCommandEvent& event);
    void OnMenuItemNormCheckSelected(wxCommandEvent& event);
    void OnMenuItemNeighCheckSelected(wxCommandEvent& event);
    void OnShowLightsSelected(wxCommandEvent& event);
    void OnMenuItemShowEdgesSelected(wxCommandEvent& event);
    void OnShowWakeSelected(wxCommandEvent& event);
    void OnMenuItemShowNodesSelected(wxCommandEvent& event);
    void OnMenuItemShowCoordSelected(wxCommandEvent& event);
    void OnMenuItemShowLegendSelected(wxCommandEvent& event);
    void OnMenuItemShowNodeIDSelected(wxCommandEvent& event);
    void OnMenuItemShowElemIDSelected(wxCommandEvent& event);
    void OnMenuItemSelResSelected(wxCommandEvent& event);
    void OnMenuItemAnalysisReportSelected(wxCommandEvent& event);
    void OnMenuItemDeleteNodesSelected(wxCommandEvent& event);
    void OnMenuItemDeleteElementsSelected(wxCommandEvent& event);
    void OnMenuItemDeleteNeighborsSelected(wxCommandEvent& event);
    void OnMenuItemDeleteResultsSelected(wxCommandEvent& event);
    void OnMenuItemDeleteMeshSelected(wxCommandEvent& event);
    void OnMenuItemGeneralSettings(wxCommandEvent& event);
    void OnAbout(wxCommandEvent& event);
    void OnToolBarItemRefitClicked(wxCommandEvent& event);
    void OnToolBarItemFrontClicked(wxCommandEvent& event);
    void OnToolBarItemTopClicked(wxCommandEvent& event);
    void OnToolBarItemRightClicked(wxCommandEvent& event);
    void OnToolBarItemBackClicked(wxCommandEvent& event);
    void OnToolBarItemBottomClicked(wxCommandEvent& event);
    void OnToolBarItemLeftClicked(wxCommandEvent& event);
    void OnToolBarItemIsometricClicked(wxCommandEvent& event);
    void OnMenuItemClearSelectionsSelected(wxCommandEvent& event);
    void OnMenuItemClearNodeSelectionSelected(wxCommandEvent& event);
    void OnMenuItemClearElementSelectionSelected(wxCommandEvent& event);
    void OnMenuItemPopEditNodesSelected(wxCommandEvent& event);
    void OnMenuItemPopEditElementsSelected(wxCommandEvent& event);
    void OnMenuItemPopDeleteNodesSelected(wxCommandEvent& event);
    void OnMenuItemPopDeleteElementsSelected(wxCommandEvent& event);
    void OnMenuItemPopReverseElementsSelected(wxCommandEvent& event);
    void OnMenuItemSelNodeSelected(wxCommandEvent& event);
    void OnMenuItemSelElemSelected(wxCommandEvent& event);
    void OnGLCanvasLeftMouse(wxMouseEvent& event);
    void OnGLCanvasRightMouse(wxMouseEvent& event);
    void OnGLCanvasMiddleUp(wxMouseEvent& event);
    void OnGLCanvasMiddleDown(wxMouseEvent& event);
    void OnGLCanvasMouseWheel(wxMouseEvent& event);
    void OnGLCanvasMouseMove(wxMouseEvent& event);
    void OnResize(wxSizeEvent& event);
    void OnPaint(wxPaintEvent& event);
    //*)

    //(*Identifiers(ApameGUIFrame)
    static const long ID_GLCANVAS1;
    static const long idMenuNew;
    static const long idMenuOpen;
    static const long idMenuSave;
    static const long idMenuSaveAs;
    static const long idMenuApameIn;
    static const long idMenuApameRes;
    static const long idMenuABQIn;
    static const long idMenuFluentMsh;
    static const long idMenuIdeas;
    static const long idMenuNastran;
    static const long idMenuVTKIn;
    static const long idMenuImport;
    static const long idMenuApameOut;
    static const long idMenuABQ;
    static const long idMenuNAS;
    static const long idMenuVTK;
    static const long idMenuExport;
    static const long idMenuQuit;
    static const long idMenuUndo;
    static const long idMenuRedo;
    static const long idMenuSelectNodes;
    static const long idMenuSelectElems;
    static const long idMenuScale;
    static const long idMenuTranslate;
    static const long idMenuRotate;
    static const long idMenuMirror;
    static const long idMenuChangeElemType;
    static const long idMenuWakeElems;
    static const long idMenuNode;
    static const long idMenuElement;
    static const long idMenuNeighbors;
    static const long idMenuWake;
    static const long idMenuExtrude;
    static const long idMenuSeparate;
    static const long ID_MENUITEM6;
    static const long ID_MENUITEM9;
    static const long ID_MENUITEM12;
    static const long ID_MENUITEM21;
    static const long ID_MENUITEM22;
    static const long idMenuFlowParams;
    static const long idMenuRefs;
    static const long idMenuSolvParams;
    static const long idMenuResReq;
    static const long idMenuRun;
    static const long idMenuDispWire;
    static const long idMenuDispSolid;
    static const long idMenuNormCheck;
    static const long idMenuNeighCheck;
    static const long idMenuLights;
    static const long idMenuDispEdge;
    static const long ID_MENUITEM4;
    static const long idMenuNodes;
    static const long idMenuCoord;
    static const long idMenuLegend;
    static const long idMenuShowNodeID;
    static const long idMenuShowElemID;
    static const long ID_MENUITEM23;
    static const long ID_MENUITEM24;
    static const long idMenuDelNode;
    static const long idMenuDelElem;
    static const long idMenuDelNeigh;
    static const long idMenuDelRes;
    static const long idMenuDelMesh;
    static const long idMenuGeneral;
    static const long idMenuAbout;
    static const long ID_STATUSBAR1;
    static const long IdNew;
    static const long idOpen;
    static const long IdSave;
    static const long idUndo;
    static const long idRedo;
    static const long idRefit;
    static const long idFront;
    static const long idTop;
    static const long ID_TOOLBARITEM1;
    static const long idBack;
    static const long idBottom;
    static const long idLeft;
    static const long idIsometric;
    static const long idRun;
    static const long ID_TOOLBAR1;
    static const long ID_MENUITEM10;
    static const long ID_MENUITEM14;
    static const long ID_MENUITEM15;
    static const long ID_MENUITEM16;
    static const long ID_MENUITEM11;
    static const long ID_MENUITEM17;
    static const long ID_MENUITEM18;
    static const long ID_MENUITEM19;
    static const long ID_MENUITEM20;
    static const long ID_MENUITEM13;
    static const long ID_MENUITEM2;
    static const long ID_MENUITEM3;
    static const long ID_MENUITEM1;
    //*)

    //(*Declarations(ApameGUIFrame)
    wxMenuItem* MenuItemDeleteResults;
    wxMenuItem* MenuItemSurfacePlot;
    wxMenuItem* MenuItemGeneralSettings;
    wxMenuItem* MenuItem8;
    wxMenuItem* MenuItemTranslate;
    wxMenuItem* MenuItemShowEdges;
    wxToolBar* ToolBar1;
    wxMenuItem* MenuItemSave;
    wxToolBarToolBase* ToolBarItem9;
    wxMenuItem* MenuItem7;
    wxMenuItem* MenuItemEditWakes;
    wxMenuItem* MenuItemSaveAs;
    wxMenuItem* MenuItemShowElemID;
    wxMenuItem* MenuItemShowLegend;
    wxToolBarToolBase* ToolBarItem12;
    wxFileDialog* ApameGUISaveFileDialog;
    wxMenuItem* MenuItemRun;
    wxMenuItem* MenuItemExtrudeLine;
    wxMenuItem* MenuItemElem;
    wxMenuItem* MenuItemPopEditElements;
    wxToolBarToolBase* ToolBarItem11;
    wxMenuItem* MenuItemNeighCheck;
    wxMenuItem* MenuItemRotate;
    wxFileDialog* ApameGUIOpenFileDialog;
    wxMenuItem* MenuItemSelectElems;
    wxMenuItem* MenuItemMergeNodes;
    wxMenu* Menu3;
    wxMenuItem* MenuItemSelectNodes;
    wxMenuItem* MenuItemClearNodeSelection;
    wxToolBarToolBase* ToolBarItem10;
    wxFileDialog* ApameSolverExportDialog;
    wxMenu* Menu10;
    wxMenu* MenuItem3;
    wxMenuItem* MenuItemMirror;
    wxMenuItem* MenuItem1;
    wxMenuItem* MenuItem4;
    wxMenuItem* MenuItemPopEditNodes;
    C3DViewer* m_p3DViewer;
    wxMenuItem* MenuItemCalcSurf;
    wxMenuItem* MenuItemClearAll;
    wxMenuItem* MenuItemShowCoord;
    wxToolBarToolBase* ToolBarItemSave;
    wxMenuItem* MenuItemDeleteNodes;
    wxMenuItem* MenuItemShowWake;
    wxMenu* MenuItem10;
    wxMenuItem* MenuItemAnalysisReport;
    wxMenuItem* MenuItemNeighbors;
    wxMenuItem* MenuItemWake;
    wxMenuItem* MenuItemRedo;
    wxMenuItem* MenuItemDeleteElements;
    wxMenuItem* MenuItemDSeparateNeighbors;
    wxMenu* MenuItem13;
    wxMenuItem* MenuItemOpen;
    wxToolBarToolBase* ToolBarItem6;
    wxMenuItem* MenuItemSolid;
    wxToolBarToolBase* ToolBarItem13;
    wxMenuItem* MenuItemExportAbaqus;
    wxMenuItem* MenuItemSelElem;
    wxToolBarToolBase* ToolBarItem1;
    wxMenuItem* MenuItemPopDeleteNodes;
    wxMenuItem* MenuItemNormCheck;
    wxMenuItem* MenuItemChangeElemType;
    wxMenuItem* MenuItemSelNode;
    wxMenuItem* MenuItemDeleteNeighbors;
    wxMenuItem* MenuItemPopDeleteElements;
    wxStatusBar* StatusBar1;
    wxMenuItem* MenuItemElemDist;
    wxMenuItem* MenuItem6;
    wxMenuItem* MenuItemNode;
    wxMenuItem* MenuItemShowLights;
    wxMenuItem* MenuItemExportNastran;
    wxMenuItem* MenuItemPopReverseElements;
    wxMenuItem* MenuItemShowNodeID;
    wxMenuItem* MenuItemScale;
    wxMenuItem* MenuItemClearElementSelection;
    wxToolBarToolBase* ToolBarItemRedo;
    wxToolBarToolBase* ToolBarItem8;
    wxToolBarToolBase* ToolBarItemRun;
    wxMenuItem* MenuItemShowNodes;
    wxMenu* Menu6;
    wxMenuItem* MenuItem9;
    wxMenuItem* MenuItemVTKExport;
    wxMenu ViewerPopUpMenu;
    wxMenuItem* MenuItemDeleteMesh;
    wxToolBarToolBase* ToolBarItem2;
    wxMenuItem* MenuItemNodeDist;
    wxMenuItem* MenuItemWireframe;
    wxMenu* Menu5;
    wxMenuItem* MenuItemBoundBox;
    wxMenuItem* MenuItemUndo;
    wxMenu* Menu4;
    wxMenu* MenuItem11;
    wxToolBarToolBase* ToolBarItemUndo;
    wxToolBarToolBase* ToolBarItem7;
    //*)

    DECLARE_EVENT_TABLE()
};

#endif
