#ifndef CGUIDATA_H
#define CGUIDATA_H

#include <wx/stdpaths.h>

#include "DSimpleMessage.h"
#include "generalFunctions.h"
#include "typenames.h"

static const int apameGUIVersion     = 0;      // ApameGUI version
static const int apameGUISubVersion  = 1;      // ApameGUI sub version
static const int apameGUIVersionDate = 140915; // ApameGUI version date

//-----------------------------------------------------------------------/
// GUI related data that needs storing to configuration file
//-----------------------------------------------------------------------/
class CGUIData
{
public:
    // general data
    string apameSolverPath;  // ApameSolver path
    string currentDir;       // current directory
    uInt useTempDir;         // use temporary directory or not
    string tempDir;          // temporary directory
    uInt logWinRefreshRate;  // ApameSolver log window refresh rate (in milliseconds)
    uInt width;              // screen width
    uInt height;             // screen height
    uInt maximized;          // flag program maximized or not
    // neighbors dialog data
    double neighThreshold;   // angle threshold for separating neighbors - angle between two elements [deg]
    // wake dialog data
    double wakeThreshold;    // angle threshold for applying Kutta condition - trailing edge sharpness [deg]
    double wakeLength;       // length of wake (in multiples of geometry span)
    uInt wakeNum;            // downstream number of wake elements

    CGUIData();
    void readGUIData();
    void storeGUIData();

private:
    void readInteger(
            uInt& storage,
            uInt& lineIndex,
            const vecStr& fileLines);
    void readDouble(
            double& storage,
            uInt& lineIndex,
            const vecStr& fileLines);
    void readString(
            string& storage,
            uInt& lineIndex,
            const vecStr& fileLines);
    int checkEndOfFile(
            const uInt& numSkip,
            const uInt& lineIndex,
            const uInt& numLines);
};

#endif
