#ifndef CPROJECTDATA_H
#define CPROJECTDATA_H

#include <wx/stdpaths.h>
#include <wx/glcanvas.h>

#include <iomanip>

#include "CGUIData.h"
#include "DSimpleMessage.h"
#include "generalFunctions.h"
#include "typenames.h"

//-----------------------------------------------------------------------/
// project session data stored in ApameGUI project file
//-----------------------------------------------------------------------/
class CProjectData
{
public:
    string projectName;
    // ApameSolver data
    double airspeed, density, pressure, mach; // flow parameters
    vecDbl alfa, beta;
    double wingspan, mac, surface;            // reference data
    vecDbl origin;
    int method, collCalc, velOrder;           // solver parameters
    double error, coplAng, collDist, farField;
    bool results, coefficients, forces, geometry, velocity, pressureCoefficient, centerPoints, doubletValues; // result requests
    bool sourceValues, velocityComponents, meshCharacteristics, staticPressure, dynamicPressure, manometerPressure;
    // surface plot dialog data
    bool surfResultsShown; // flag if surface results are shown or not
    string selectedData;   // selected surface data
    string selectedCase;   // selected case
    string plotType;       // plot type; elemental or nodal
    string limitType;      // type of data limit; automatic or manual
    double minLimit;       // lower data limit
    double maxLimit;       // upper data limit
    // 3D Viewer data
    int displayMode;       // display mode (0-wireframe; 1-solid; 2-normal; 3-neighbors)
    bool showWake;         // toggle wake display
    bool showNodes;        // toggle nodes display
    bool showLights;       // toggle lights display
    bool showEdges;        // toggle edges display
    bool showCoordSys;     // toggle coordinate system display
    bool showNodeIDs;      // toggle node IDs display
    bool showElemIDs;      // toggle element IDs display
    bool showLegend;       // toggle legend display

    CProjectData();

    int openApameGUIFile(
            string& apameGUIFileName,
            CGUIData* pGUIData,
            CMesh* pMesh,
            CResults* pResults);
    int saveApameGUIFile(
            string& apameGUIFileName,
            CMesh* pMesh,
            CResults* pResults);

private:
    int checkEndOfFile(
            const uInt& numSkip,
            const uInt& lineIndex,
            const uInt& numLines);
    int readString(
            string& storage,
            uInt& lineIndex,
            const vecStr& fileLines);
    int readBool(
            bool& storage,
            uInt& lineIndex,
            const vecStr& fileLines);
    int readInteger(
            int& storage,
            uInt& lineIndex,
            const vecStr& fileLines);
    int readUInt(
            uInt& storage,
            uInt& lineIndex,
            const vecStr& fileLines);
    int readFloat(
            double& storage,
            uInt& lineIndex,
            const vecStr& fileLines);
};

#endif
