#ifndef CRESULTS_H_INCLUDED
#define CRESULTS_H_INCLUDED

#include "CMesh.h"
#include "DSimpleMessage.h"
#include "generalFunctions.h"
#include "params.h"
#include "typenames.h"

//-----------------------------------------------------------------------/
// structure that stores surface data
//-----------------------------------------------------------------------/
struct SSurfaceData
{
    string dataName;   // name of the data
    vecStr caseTitles; // case titles
    vecUInt elemIDs;   // IDs of elements on which data is defined
    vecVecDbl data;    // result data on elements
};

class CResults
{
public:
    // solver run data
    string date, time, solverVersion;           // date, time and version of solver run
    uInt numElem, numBodyElem, numNode;         // number of nodes and elements
    double airspeed, density, pressure, mach;   // reference speed, density, pressure and MACH number
    uInt caseNumber;                            // number of cases
    vecDbl alfa, beta;                          // angles of attack and sideslip angles
    double wingspan, mac, surface;              // reference wingspan, mean aerodynamic chord (length) and surface
    vecDbl origin;                              // origin coordinates for calculating moments
    uInt method;                                // velocity interpolation method
    double error, coplAng, collDist, farField;  // smallest value, coplanarity angle, collocation distance, far-field distance
    uInt collCalc, velOrder;                    // collocation calculation method, velocity interpolation order
    // coefficients of forces and moments
    vecDbl Cx, Cy, Cz, Cl, Cm, Cn;  // in body reference coordinate system
    vecDbl Cdrag, Cside, Clift;     // in aerodynamic coordinate system
    // forces and moments
    vecDbl Fx, Fy, Fz, Fl, Fm, Fn;  // in body reference coordinate system
    vecDbl Fdrag, Fside, Flift;     // in aerodynamic coordinate system
    // surface data
    vector<SSurfaceData> surfDatas; // all available surface data

    CResults();
    int getFieldData(
        const string& dataName,
        const string& caseName,
        vecDbl& fieldData,
        vecUInt& elemIDs);
};

#endif
