#ifndef IMPORTEXPORT_H
#define IMPORTEXPORT_H

#include <fstream>

#include "CMesh.h"
#include "CProjectData.h"
#include "CResults.h"
#include "DSimpleMessage.h"
#include "generalFunctions.h"
#include "typenames.h"

void readApameSolverInputFile(
        CMesh* pMesh,
        CProjectData* pProjectData,
        const string& fileName,
        bool readFlowParams,
        bool readRefData,
        bool readSolverParams,
        bool readResReq,
        bool readNodes,
        bool readElements);
void readApameSolverResultMeshAndProjectData(
        CMesh* pMesh,
        CProjectData* pProjectData,
        const string& fileName,
        bool readFlowParams,
        bool readRefData,
        bool readSolverParams,
        bool readNodes,
        bool readElements,
        vecUInt& elemIDs);
int readApameSolverResults(
        CResults* pResults,
        const string& resFileName,
        vecUInt& elemIDs);
int readInteger(
        int& storage,
        uInt& lineIndex,
        const vecStr& fileLines);
int readUInt(
        uInt& storage,
        uInt& lineIndex,
        const vecStr& fileLines);
int readFloat(
        double& storage,
        uInt& lineIndex,
        const vecStr& fileLines);
int readCoefForce(
        vecDbl& storage,
        uInt& lineIndex,
        const uInt& caseNum,
        const vecStr& fileLines);
int readSurfaceData(
        CResults* pResults,
        const string& dataName,
        const vecUInt& elemIDs,
        uInt& lineIndex,
        const vecStr& fileLines,
        const uInt& caseNum=1,
        const vecStr& caseTitles=vecStr());
void readAbaqusFile(
        CMesh* pMesh,
        const string& fileName,
        const long& shiftNodeIDs,
        const long& shiftElemIDs);
void readFluentFile(
        CMesh* pMesh,
        const string& fileName,
        const long& shiftNodeIDs,
        const long& shiftElemIDs);
void readIdeasFile(
        CMesh* pMesh,
        const string& fileName,
        const long& shiftNodeIDs,
        const long& shiftElemIDs);
void readNastranFile(
        CMesh* pMesh,
        const string& fileName,
        const long& shiftNodeIDs,
        const long& shiftElemIDs);
double readNastranSci(const string& nastranNumber);
void readVTKFile(
        CMesh* pMesh,
        const string& fileName,
        const long& shiftNodeIDs,
        const long& shiftElemIDs);
int writeApameSolverInput(
        const string& apameFileName,
        CMesh* pMesh,
        CProjectData* pProjectData);
int exportAbaqusFile(
        const string& fileName,
        CMesh* pMesh,
        CResults* pResults,
        const bool& writeMesh,
        const bool& writeProperty,
        const bool& writeLoads,
        const string& modulus,
        const string& poisson,
        const string& thickness,
        const string& selectedData,
        const string& selectedCase);
int exportNastranFile(
        const string& fileName,
        CMesh* pMesh,
        CResults* pResults,
        const bool& writeMesh,
        const bool& writeProperty,
        const bool& writeLoads,
        const string& modulus,
        const string& poisson,
        const string& thickness,
        const string& selectedData,
        const string& selectedCase);
int exportVTKFile(
        const string& fileName,
        CMesh* pMesh,
        CResults* pResults,
        const bool& plotResults,
        const string& plotType);

#endif
