#include "wx_pch.h"
#include "DAnalysisReport.h"

#ifndef WX_PRECOMP
	//(*InternalHeadersPCH(DAnalysisReport)
	#include <wx/intl.h>
	#include <wx/string.h>
	//*)
#endif
//(*InternalHeaders(DAnalysisReport)
#include <wx/font.h>
//*)

//(*IdInit(DAnalysisReport)
const long DAnalysisReport::ID_TEXTCTRL1 = wxNewId();
const long DAnalysisReport::ID_BUTTON1 = wxNewId();
//*)

BEGIN_EVENT_TABLE(DAnalysisReport,wxDialog)
	//(*EventTable(DAnalysisReport)
	//*)
END_EVENT_TABLE()

DAnalysisReport::DAnalysisReport(
        wxWindow* parent,
        CResults* pResults,
        wxWindowID id,
        const wxPoint& pos,
        const wxSize& size) : m_pResults(pResults)
{
	//(*Initialize(DAnalysisReport)
	wxFlexGridSizer* FlexGridSizer1;

	Create(parent, id, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE, _T("id"));
	SetClientSize(wxDefaultSize);
	Move(wxDefaultPosition);
	FlexGridSizer1 = new wxFlexGridSizer(2, 1, 0, 0);
	AnalysisText = new wxTextCtrl(this, ID_TEXTCTRL1, wxEmptyString, wxDefaultPosition, wxSize(800,600), wxTE_MULTILINE|wxTE_READONLY, wxDefaultValidator, _T("ID_TEXTCTRL1"));
	wxFont AnalysisTextFont(10,wxSWISS,wxFONTSTYLE_NORMAL,wxNORMAL,false,_T("Courier New"),wxFONTENCODING_DEFAULT);
	AnalysisText->SetFont(AnalysisTextFont);
	FlexGridSizer1->Add(AnalysisText, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	CloseButton = new wxButton(this, ID_BUTTON1, _("Close"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_BUTTON1"));
	FlexGridSizer1->Add(CloseButton, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	SetSizer(FlexGridSizer1);
	FlexGridSizer1->Fit(this);
	FlexGridSizer1->SetSizeHints(this);

	Connect(ID_BUTTON1,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&DAnalysisReport::OnCloseButtonClick);
	//*)
    
    // exit if no results exist
    if (!m_pResults)
        return;

    // print general info
    *AnalysisText << "GENERAL INFO:\n\n";
    *AnalysisText << "Date of analysis      : " << m_pResults->date << "\n";
    *AnalysisText << "Time of analysis      : " << m_pResults->time << "\n";
    *AnalysisText << "Solver version        : " << m_pResults->solverVersion << "\n";
    *AnalysisText << "Number of body panels : " << int2Str(m_pResults->numElem) << "\n";
    *AnalysisText << "Number of wake panels : " << int2Str(m_pResults->numElem - m_pResults->numBodyElem) << "\n";
    *AnalysisText << "Number of nodes       : " << int2Str(m_pResults->numNode) << "\n";
    *AnalysisText << "\n\n";

    // print flow parameters
    *AnalysisText << "FLOW PARAMETERS:"                             << "\n\n";
    *AnalysisText << "Airspeed        : " << m_pResults->airspeed   << "\n";
    *AnalysisText << "Density         : " << m_pResults->density    << "\n";
    *AnalysisText << "Pressure        : " << m_pResults->pressure   << "\n";
    *AnalysisText << "Mach number     : " << m_pResults->mach       << "\n";
    *AnalysisText << "Number of cases : " << int2Str(m_pResults->caseNumber) << "\n";
    *AnalysisText << "    Angles of Attack:";
    for (uInt i=0; i<m_pResults->caseNumber; i++)
        *AnalysisText << " " << m_pResults->alfa[i];
    *AnalysisText << "\n";
    *AnalysisText << "    Sideslip Angles: ";
    for (uInt i=0; i<m_pResults->caseNumber; i++)
        *AnalysisText << " " << m_pResults->beta[i];
    *AnalysisText << "\n";
    *AnalysisText << "\n\n";

    // print reference values
    *AnalysisText << "REFERENCE VALUES:" << "\n\n";
    *AnalysisText << "Wingspan : " << m_pResults->wingspan  << "\n";
    *AnalysisText << "Mac      : " << m_pResults->mac       << "\n";
    *AnalysisText << "Surface  : " << m_pResults->surface   << "\n";
    if (m_pResults->origin.size() == 3)
        *AnalysisText << "Origin   : " << m_pResults->origin[0] << " "
                                      << m_pResults->origin[1] << " "
                                      << m_pResults->origin[2] << "\n";
    *AnalysisText << "\n\n";

    // print solver parameters
    *AnalysisText << "SOLVER PARAMETERS:" << "\n\n";
    if      (m_pResults->method == 0) *AnalysisText << "Solver method                  : Constant Source/Doublet" << "\n";
    else if (m_pResults->method == 1) *AnalysisText << "Solver method                  : Constant Doublet"        << "\n";
    *AnalysisText << "Error value                    : " << m_pResults->error    << "\n";
    *AnalysisText << "Coplanarity Angle              : " << m_pResults->coplAng  << "\n";
    *AnalysisText << "Collocation distance           : " << m_pResults->collDist << "\n";
    *AnalysisText << "Far-field value                : " << m_pResults->farField << "\n";
    if      (m_pResults->collCalc == 0) *AnalysisText << "Collocation calculation method : Approximative" << "\n";
    else if (m_pResults->collCalc == 1) *AnalysisText << "Collocation calculation method : Accurate"      << "\n";
    if      (m_pResults->velOrder == 0) *AnalysisText << "Velocity interpolation         : Nodal"         << "\n";
    else if (m_pResults->velOrder == 1) *AnalysisText << "Velocity interpolation         : First order"   << "\n";
    else if (m_pResults->velOrder == 2) *AnalysisText << "Velocity interpolation         : Second order"  << "\n";
    *AnalysisText << "\n\n";

    // print coefficients
    *AnalysisText << "COEFFICIENTS:" << "\n\n";
    *AnalysisText << " ALFA              ";
    for (uInt i=0; i<m_pResults->caseNumber; i++)
        *AnalysisText << "   " << dbl2StrCase(m_pResults->alfa[i]);
    *AnalysisText << "\n";
    *AnalysisText << " BETA              ";
    for (uInt i=0; i<m_pResults->caseNumber; i++)
        *AnalysisText << "   " << dbl2StrCase(m_pResults->beta[i]);
    *AnalysisText << "\n";
    *AnalysisText << " CX                 ";
    for (uInt i=0; i<m_pResults->Cx.size(); i++) *AnalysisText << " " << dbl2StrCoeff(m_pResults->Cx[i]);
    *AnalysisText << "\n";
    *AnalysisText << " CY                 ";
    for (uInt i=0; i<m_pResults->Cy.size(); i++) *AnalysisText << " " << dbl2StrCoeff(m_pResults->Cy[i]);
    *AnalysisText << "\n";
    *AnalysisText << " CZ                 ";
    for (uInt i=0; i<m_pResults->Cz.size(); i++) *AnalysisText << " " << dbl2StrCoeff(m_pResults->Cz[i]);
    *AnalysisText << "\n";
    *AnalysisText << " CL                 ";
    for (uInt i=0; i<m_pResults->Cl.size(); i++) *AnalysisText << " " << dbl2StrCoeff(m_pResults->Cl[i]);
    *AnalysisText << "\n";
    *AnalysisText << " CM                 ";
    for (uInt i=0; i<m_pResults->Cm.size(); i++) *AnalysisText << " " << dbl2StrCoeff(m_pResults->Cm[i]);
    *AnalysisText << "\n";
    *AnalysisText << " CN                 ";
    for (uInt i=0; i<m_pResults->Cn.size(); i++) *AnalysisText << " " << dbl2StrCoeff(m_pResults->Cn[i]);
    *AnalysisText << "\n";
    *AnalysisText << " Drag coefficient   ";
    for (uInt i=0; i<m_pResults->Cdrag.size(); i++) *AnalysisText << " " << dbl2StrCoeff(m_pResults->Cdrag[i]);
    *AnalysisText << "\n";
    *AnalysisText << " Side coefficient   ";
    for (uInt i=0; i<m_pResults->Cside.size(); i++) *AnalysisText << " " << dbl2StrCoeff(m_pResults->Cside[i]);
    *AnalysisText << "\n";
    *AnalysisText << " Lift coefficient   ";
    for (uInt i=0; i<m_pResults->Clift.size(); i++) *AnalysisText << " " << dbl2StrCoeff(m_pResults->Clift[i]);
    *AnalysisText << "\n";
    *AnalysisText << "\n\n";

    // print forces
    *AnalysisText << "FORCES:" << "\n\n";
    *AnalysisText << " ALFA      ";
    for (uInt i=0; i<m_pResults->caseNumber; i++)
        *AnalysisText << "           " << dbl2StrCase(m_pResults->alfa[i]);
    *AnalysisText << "\n";
    *AnalysisText << " BETA      ";
    for (uInt i=0; i<m_pResults->caseNumber; i++)
        *AnalysisText << "           " << dbl2StrCase(m_pResults->beta[i]);
    *AnalysisText << "\n";
    *AnalysisText << " FX                 ";
    for (uInt i=0; i<m_pResults->Fx.size(); i++) *AnalysisText << " " << dbl2StrForc(m_pResults->Fx[i]);
    *AnalysisText << "\n";
    *AnalysisText << " FY                 ";
    for (uInt i=0; i<m_pResults->Fy.size(); i++) *AnalysisText << " " << dbl2StrForc(m_pResults->Fy[i]);
    *AnalysisText << "\n";
    *AnalysisText << " FZ                 ";
    for (uInt i=0; i<m_pResults->Fz.size(); i++) *AnalysisText << " " << dbl2StrForc(m_pResults->Fz[i]);
    *AnalysisText << "\n";
    *AnalysisText << " FL                 ";
    for (uInt i=0; i<m_pResults->Fl.size(); i++) *AnalysisText << " " << dbl2StrForc(m_pResults->Fl[i]);
    *AnalysisText << "\n";
    *AnalysisText << " FM                 ";
    for (uInt i=0; i<m_pResults->Fm.size(); i++) *AnalysisText << " " << dbl2StrForc(m_pResults->Fm[i]);
    *AnalysisText << "\n";
    *AnalysisText << " FN                 ";
    for (uInt i=0; i<m_pResults->Fn.size(); i++) *AnalysisText << " " << dbl2StrForc(m_pResults->Fn[i]);
    *AnalysisText << "\n";
    *AnalysisText << " Drag force         ";
    for (uInt i=0; i<m_pResults->Fdrag.size(); i++) *AnalysisText << " " << dbl2StrForc(m_pResults->Fdrag[i]);
    *AnalysisText << "\n";
    *AnalysisText << " Side force         ";
    for (uInt i=0; i<m_pResults->Fside.size(); i++) *AnalysisText << " " << dbl2StrForc(m_pResults->Fside[i]);
    *AnalysisText << "\n";
    *AnalysisText << " Lift force         ";
    for (uInt i=0; i<m_pResults->Flift.size(); i++) *AnalysisText << " " << dbl2StrForc(m_pResults->Flift[i]);
    *AnalysisText << "\n";
}

DAnalysisReport::~DAnalysisReport()
{
	//(*Destroy(DAnalysisReport)
	//*)
}

void DAnalysisReport::OnCloseButtonClick(wxCommandEvent& event)
{
    this->Destroy();
}
