#include "wx_pch.h"
#include "DDeleteNodes.h"

#ifndef WX_PRECOMP
	//(*InternalHeadersPCH(DDeleteNodes)
	#include <wx/intl.h>
	#include <wx/string.h>
	//*)
#endif
//(*InternalHeaders(DDeleteNodes)
//*)

//(*IdInit(DDeleteNodes)
const long DDeleteNodes::ID_STATICTEXT1 = wxNewId();
const long DDeleteNodes::ID_RADIOBOX1 = wxNewId();
const long DDeleteNodes::ID_BUTTON3 = wxNewId();
const long DDeleteNodes::ID_BUTTON1 = wxNewId();
const long DDeleteNodes::ID_BUTTON2 = wxNewId();
//*)

BEGIN_EVENT_TABLE(DDeleteNodes,wxDialog)
	//(*EventTable(DDeleteNodes)
	//*)
END_EVENT_TABLE()

DDeleteNodes::DDeleteNodes(
        wxWindow* parent,
        bool& modified,
        CMesh* pMesh,
        setUInt& nodeSelection,
        wxWindowID id,
        const wxPoint& pos,
        const wxSize& size) : m_modified(modified), m_pMesh(pMesh), m_nodeSelection(nodeSelection)
{
	//(*Initialize(DDeleteNodes)
	wxFlexGridSizer* FlexGridSizer4;
	wxFlexGridSizer* FlexGridSizer3;
	wxFlexGridSizer* FlexGridSizer2;
	wxFlexGridSizer* FlexGridSizer1;

	Create(parent, wxID_ANY, _("Delete Nodes"), wxDefaultPosition, wxDefaultSize, wxSTAY_ON_TOP|wxCAPTION, _T("wxID_ANY"));
	FlexGridSizer1 = new wxFlexGridSizer(3, 1, 0, 0);
	FlexGridSizer2 = new wxFlexGridSizer(0, 3, 0, 0);
	StaticText1 = new wxStaticText(this, ID_STATICTEXT1, _("NOTE: Nodes used for element construction will be skipped"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT1"));
	FlexGridSizer2->Add(StaticText1, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	FlexGridSizer1->Add(FlexGridSizer2, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	FlexGridSizer3 = new wxFlexGridSizer(0, 3, 0, 0);
	wxString __wxRadioBoxChoices_1[2] =
	{
		_("All Nodes"),
		_("Node Selection")
	};
	DomainRadioBox = new wxRadioBox(this, ID_RADIOBOX1, _("Select Domain"), wxDefaultPosition, wxDefaultSize, 2, __wxRadioBoxChoices_1, 1, 0, wxDefaultValidator, _T("ID_RADIOBOX1"));
	DomainRadioBox->SetSelection(0);
	FlexGridSizer3->Add(DomainRadioBox, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	SelectButton = new wxButton(this, ID_BUTTON3, _("Select"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_BUTTON3"));
	SelectButton->Disable();
	FlexGridSizer3->Add(SelectButton, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	FlexGridSizer1->Add(FlexGridSizer3, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	FlexGridSizer4 = new wxFlexGridSizer(0, 3, 0, 0);
	OKButton = new wxButton(this, ID_BUTTON1, _("OK"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_BUTTON1"));
	FlexGridSizer4->Add(OKButton, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	CancelButton = new wxButton(this, ID_BUTTON2, _("Cancel"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_BUTTON2"));
	FlexGridSizer4->Add(CancelButton, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	FlexGridSizer1->Add(FlexGridSizer4, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	SetSizer(FlexGridSizer1);
	FlexGridSizer1->Fit(this);
	FlexGridSizer1->SetSizeHints(this);

	Connect(ID_RADIOBOX1,wxEVT_COMMAND_RADIOBOX_SELECTED,(wxObjectEventFunction)&DDeleteNodes::OnDomainRadioBoxSelect);
	Connect(ID_BUTTON3,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&DDeleteNodes::OnSelectButtonClick);
	Connect(ID_BUTTON1,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&DDeleteNodes::OnOKButtonClick);
	Connect(ID_BUTTON2,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&DDeleteNodes::OnCancelButtonClick);
	//*)
}

DDeleteNodes::~DDeleteNodes()
{
	//(*Destroy(DDeleteNodes)
	//*)
}

void DDeleteNodes::OnOKButtonClick(wxCommandEvent& event)
{
    m_modified = true;

    // check if all nodes or selection requested for deletion
    if (DomainRadioBox->GetStringSelection() == _("All Nodes"))
        m_pMesh->deleteAllNodes();
    else{
        // assemble node ID's from selection
        vecUInt nodeIDs(m_nodeSelection.size());
        uInt i = 0;
        for (setUIntIt it=m_nodeSelection.begin(); it!=m_nodeSelection.end(); it++)
            nodeIDs[i++] = *it;

        // delete these elements
        m_pMesh->deleteNodesByID(nodeIDs);
    }

    // finish
    this->Destroy();
}

void DDeleteNodes::OnCancelButtonClick(wxCommandEvent& event)
{
    this->Destroy();
}

void DDeleteNodes::OnSelectButtonClick(wxCommandEvent& event)
{
    DNodeSelection* node = new DNodeSelection(0, m_pMesh, m_nodeSelection);
    node->ShowModal();
}

void DDeleteNodes::OnDomainRadioBoxSelect(wxCommandEvent& event)
{
    // enable/disable select button
    if (DomainRadioBox->GetStringSelection() == _("All Nodes"))
        SelectButton->Enable(false);
    else
        SelectButton->Enable(true);
}
