#include "wx_pch.h"
#include "DEditWakes.h"

#ifndef WX_PRECOMP
	//(*InternalHeadersPCH(DEditWakes)
	#include <wx/intl.h>
	#include <wx/string.h>
	//*)
#endif
//(*InternalHeaders(DEditWakes)
//*)

//(*IdInit(DEditWakes)
const long DEditWakes::ID_STATICTEXT1 = wxNewId();
const long DEditWakes::ID_TEXTCTRL1 = wxNewId();
const long DEditWakes::ID_STATICTEXT2 = wxNewId();
const long DEditWakes::ID_TEXTCTRL2 = wxNewId();
const long DEditWakes::ID_BUTTON3 = wxNewId();
const long DEditWakes::ID_BUTTON1 = wxNewId();
const long DEditWakes::ID_BUTTON2 = wxNewId();
//*)

BEGIN_EVENT_TABLE(DEditWakes,wxDialog)
	//(*EventTable(DEditWakes)
	//*)
END_EVENT_TABLE()

DEditWakes::DEditWakes(
        wxWindow* parent,
        bool& modified,
        CMesh* pMesh,
        setUInt& elemSelection,
        wxWindowID id,
        const wxPoint& pos,
        const wxSize& size) : m_modified(modified), m_pMesh(pMesh), m_elemSelection(elemSelection)
{
	//(*Initialize(DEditWakes)
	wxFlexGridSizer* FlexGridSizer4;
	wxFlexGridSizer* FlexGridSizer3;
	wxFlexGridSizer* FlexGridSizer2;
	wxFlexGridSizer* FlexGridSizer1;

	Create(parent, id, _("Edit Wake Elements"), wxDefaultPosition, wxDefaultSize, wxSTAY_ON_TOP|wxCAPTION, _T("id"));
	SetClientSize(wxDefaultSize);
	Move(wxDefaultPosition);
	FlexGridSizer1 = new wxFlexGridSizer(0, 1, 0, 0);
	FlexGridSizer2 = new wxFlexGridSizer(2, 2, 0, 0);
	StaticText1 = new wxStaticText(this, ID_STATICTEXT1, _("Upper Element ID"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT1"));
	FlexGridSizer2->Add(StaticText1, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	UpperElem = new wxTextCtrl(this, ID_TEXTCTRL1, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_TEXTCTRL1"));
	FlexGridSizer2->Add(UpperElem, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	StaticText2 = new wxStaticText(this, ID_STATICTEXT2, _("Lower Element ID"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT2"));
	FlexGridSizer2->Add(StaticText2, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	LowerElem = new wxTextCtrl(this, ID_TEXTCTRL2, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_TEXTCTRL2"));
	FlexGridSizer2->Add(LowerElem, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	FlexGridSizer1->Add(FlexGridSizer2, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	FlexGridSizer3 = new wxFlexGridSizer(0, 3, 0, 0);
	SelectButton = new wxButton(this, ID_BUTTON3, _("SelectElements"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_BUTTON3"));
	FlexGridSizer3->Add(SelectButton, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	FlexGridSizer1->Add(FlexGridSizer3, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	FlexGridSizer4 = new wxFlexGridSizer(0, 3, 0, 0);
	OKButton = new wxButton(this, ID_BUTTON1, _("OK"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_BUTTON1"));
	FlexGridSizer4->Add(OKButton, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	CancelButton = new wxButton(this, ID_BUTTON2, _("Cancel"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_BUTTON2"));
	FlexGridSizer4->Add(CancelButton, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	FlexGridSizer1->Add(FlexGridSizer4, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	SetSizer(FlexGridSizer1);
	FlexGridSizer1->Fit(this);
	FlexGridSizer1->SetSizeHints(this);

	Connect(ID_BUTTON3,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&DEditWakes::OnSelectButtonClick);
	Connect(ID_BUTTON1,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&DEditWakes::OnOKButtonClick);
	Connect(ID_BUTTON2,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&DEditWakes::OnCancelButtonClick);
	//*)
}

DEditWakes::~DEditWakes()
{
	//(*Destroy(DEditWakes)
	//*)
}

void DEditWakes::OnCancelButtonClick(wxCommandEvent& event)
{
    this->Destroy();
}

void DEditWakes::OnSelectButtonClick(wxCommandEvent& event)
{
    DElemSelection* elemSelection = new DElemSelection(0, m_pMesh, m_elemSelection);
    elemSelection->ShowModal();
}

void DEditWakes::OnOKButtonClick(wxCommandEvent& event)
{
    // extract trailing element IDs
    long upperID, lowerID;
    wxString number;
    number = UpperElem->GetValue();
    if(!number.ToLong(&upperID) || upperID < 1){
        showErrorMessage("Incorrect value for upper trailing element ID entered");
        return;
    }
    number = LowerElem->GetValue();
    if(!number.ToLong(&lowerID) || lowerID < 1){
        showErrorMessage("Incorrect value for lower trailing element ID entered");
        return;
    }

    m_modified = true;

    // update wakes
    int retCode = m_pMesh->updateWakeTrails(upperID, lowerID, m_elemSelection);
    if (retCode == 1)
        showWarningMessage("Non-wake element types found in element selection were ignored");
    else if (retCode == -1){
        showErrorMessage("Given trail ID(s) does not exist");
        return;
    }
    else if (retCode == -2){
        showErrorMessage("Upper and lower trails are the same");
        return;
    }
    else if (retCode == -3){
        showErrorMessage("Element cannot have itself as a trailing element");
        return;
    }

    this->Destroy();
}
