/*

VTK export functionality has been hugely done in cooperation with Julien Chaussee

*/

#include "wx_pch.h"
#include "DExportVTK.h"

#ifndef WX_PRECOMP
//(*InternalHeadersPCH(DExportVTK)
#include <wx/intl.h>
#include <wx/string.h>
//*)
#endif
//(*InternalHeaders(DExportVTK)
//*)

//(*IdInit(DExportVTK)
const long DExportVTK::ID_CHECKBOX1 = wxNewId();
const long DExportVTK::ID_RADIOBOX1 = wxNewId();
const long DExportVTK::ID_TEXTCTRL1 = wxNewId();
const long DExportVTK::ID_BUTTON1 = wxNewId();
const long DExportVTK::ID_BUTTON2 = wxNewId();
const long DExportVTK::ID_BUTTON3 = wxNewId();
//*)

BEGIN_EVENT_TABLE(DExportVTK,wxDialog)
    //(*EventTable(DExportVTK)
    //*)
END_EVENT_TABLE()

DExportVTK::DExportVTK(
        wxWindow* parent,
        CMesh* pMesh,
        CProjectData* pProjectData,
        CResults* pResults,
        wxWindowID id) : m_pMesh(pMesh), m_pProjectData(pProjectData), m_pResults(pResults)
{
    //(*Initialize(DExportVTK)
    wxFlexGridSizer* FlexGridSizer3;
    wxFlexGridSizer* FlexGridSizer2;
    wxFlexGridSizer* FlexGridSizer1;

    Create(parent, wxID_ANY, _("Export VTK File..."), wxDefaultPosition, wxDefaultSize, wxSTAY_ON_TOP|wxCAPTION, _T("wxID_ANY"));
    FlexGridSizer1 = new wxFlexGridSizer(4, 1, 0, 0);
    FlexGridSizer1->AddGrowableCol(0);
    FlexGridSizer1->AddGrowableRow(2);
    ResultsCheckBox = new wxCheckBox(this, ID_CHECKBOX1, _("Export Results"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_CHECKBOX1"));
    ResultsCheckBox->SetValue(false);
    FlexGridSizer1->Add(ResultsCheckBox, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
    wxString __wxRadioBoxChoices_1[2] =
    {
    	_("Elemental"),
    	_("Nodal")
    };
    PlotTypeRadioBox = new wxRadioBox(this, ID_RADIOBOX1, _("Plot Type"), wxDefaultPosition, wxDefaultSize, 2, __wxRadioBoxChoices_1, 1, 0, wxDefaultValidator, _T("ID_RADIOBOX1"));
    PlotTypeRadioBox->SetSelection(0);
    FlexGridSizer1->Add(PlotTypeRadioBox, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
    FlexGridSizer2 = new wxFlexGridSizer(1, 2, 0, 0);
    FlexGridSizer2->AddGrowableCol(0);
    FileName = new wxTextCtrl(this, ID_TEXTCTRL1, wxEmptyString, wxDefaultPosition, wxSize(200,-1), 0, wxDefaultValidator, _T("ID_TEXTCTRL1"));
    FlexGridSizer2->Add(FileName, 1, wxALL|wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    FileNameButton = new wxButton(this, ID_BUTTON1, _("File Name"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_BUTTON1"));
    FlexGridSizer2->Add(FileNameButton, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    FlexGridSizer1->Add(FlexGridSizer2, 1, wxALL|wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    FlexGridSizer3 = new wxFlexGridSizer(1, 2, 0, 0);
    OKButton = new wxButton(this, ID_BUTTON2, _("OK"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_BUTTON2"));
    FlexGridSizer3->Add(OKButton, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    CancelButton = new wxButton(this, ID_BUTTON3, _("Cancel"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_BUTTON3"));
    FlexGridSizer3->Add(CancelButton, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    FlexGridSizer1->Add(FlexGridSizer3, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    SetSizer(FlexGridSizer1);
    ExportFileDialog = new wxFileDialog(this, _("Select file"), wxEmptyString, wxEmptyString, _("VTK file (*.vtk)|*.vtk|All files (*.*)|*.*"), wxFD_SAVE|wxFD_OVERWRITE_PROMPT, wxDefaultPosition, wxDefaultSize, _T("wxFileDialog"));
    FlexGridSizer1->Fit(this);
    FlexGridSizer1->SetSizeHints(this);

    Connect(ID_CHECKBOX1,wxEVT_COMMAND_CHECKBOX_CLICKED,(wxObjectEventFunction)&DExportVTK::OnResultsCheckBoxClick);
    Connect(ID_BUTTON1,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&DExportVTK::OnBrowseButtonClick);
    Connect(ID_BUTTON2,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&DExportVTK::OnOKButtonClick);
    Connect(ID_BUTTON3,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&DExportVTK::OnCancelButtonClick);
    //*)

    // append job name into filename box
    FileName->AppendText(str2Wxs(m_pProjectData->projectName+".vtk"));

    // check if results exist
    if (m_pResults){
        ResultsCheckBox->Enable();
        ResultsCheckBox->SetValue(true);
        PlotTypeRadioBox->Enable();
    }
    else{
        ResultsCheckBox->Disable();
        ResultsCheckBox->SetValue(false);
        PlotTypeRadioBox->Disable();
    }
}

DExportVTK::~DExportVTK()
{
    //(*Destroy(DExportVTK)
    //*)
}

void DExportVTK::OnCancelButtonClick(wxCommandEvent& event)
{
    this->Destroy();
}

void DExportVTK::OnBrowseButtonClick(wxCommandEvent& event)
{
    // show browse dialog and get filename with full path
    ExportFileDialog->SetFilename(FileName->GetValue());
    if (ExportFileDialog->ShowModal() == wxID_OK)
        FileName->SetValue(ExportFileDialog->GetPath());
}

void DExportVTK::OnResultsCheckBoxClick(wxCommandEvent& event)
{
    if (ResultsCheckBox->IsChecked())
        PlotTypeRadioBox->Enable();
    else
        PlotTypeRadioBox->Disable();
}

void DExportVTK::OnOKButtonClick(wxCommandEvent& event)
{
    // get file name from filename box
    string fileName = wxs2Str(FileName->GetValue());

    // get plot type
    string plotType;
    if (PlotTypeRadioBox->GetStringSelection() == "Elemental")
        plotType = "elemental";
    else
        plotType = "nodal";

    // save file
    if (exportVTKFile(fileName, m_pMesh, m_pResults, ResultsCheckBox->IsChecked(), plotType))
        return;

    this->Destroy();
}
