#include "wx_pch.h"
#include "DGenerateNeighbors.h"

#ifndef WX_PRECOMP
	//(*InternalHeadersPCH(DGenerateNeighbors)
	#include <wx/intl.h>
	#include <wx/string.h>
	//*)
#endif
//(*InternalHeaders(DGenerateNeighbors)
//*)

//(*IdInit(DGenerateNeighbors)
const long DGenerateNeighbors::ID_STATICTEXT1 = wxNewId();
const long DGenerateNeighbors::ID_TEXTCTRL1 = wxNewId();
const long DGenerateNeighbors::ID_RADIOBOX1 = wxNewId();
const long DGenerateNeighbors::ID_BUTTON3 = wxNewId();
const long DGenerateNeighbors::ID_BUTTON1 = wxNewId();
const long DGenerateNeighbors::ID_BUTTON2 = wxNewId();
//*)

BEGIN_EVENT_TABLE(DGenerateNeighbors,wxDialog)
	//(*EventTable(DGenerateNeighbors)
	//*)
END_EVENT_TABLE()

DGenerateNeighbors::DGenerateNeighbors(
        wxWindow* parent,
        bool& modified,
        CGUIData* pGUIData,
        CMesh* pMesh,
        setUInt& elemSelection,
        wxWindowID id,
        const wxPoint& pos,
        const wxSize& size) : m_modified(modified), m_pGUIData(pGUIData), m_pMesh(pMesh), m_elemSelection(elemSelection)
{
	//(*Initialize(DGenerateNeighbors)
	wxFlexGridSizer* FlexGridSizer4;
	wxFlexGridSizer* FlexGridSizer3;
	wxFlexGridSizer* FlexGridSizer5;
	wxFlexGridSizer* FlexGridSizer2;
	wxFlexGridSizer* FlexGridSizer1;
	
	Create(parent, wxID_ANY, _("Find Neighbors"), wxDefaultPosition, wxDefaultSize, wxSTAY_ON_TOP|wxCAPTION, _T("wxID_ANY"));
	FlexGridSizer1 = new wxFlexGridSizer(2, 1, 0, 0);
	FlexGridSizer1->AddGrowableCol(0);
	FlexGridSizer1->AddGrowableRow(0);
	FlexGridSizer2 = new wxFlexGridSizer(2, 1, 0, 0);
	FlexGridSizer4 = new wxFlexGridSizer(1, 2, 0, 0);
	StaticText1 = new wxStaticText(this, ID_STATICTEXT1, _("Angle Threshold For Separating Neighbors"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT1"));
	FlexGridSizer4->Add(StaticText1, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	neighborThreshold = new wxTextCtrl(this, ID_TEXTCTRL1, wxEmptyString, wxDefaultPosition, wxSize(100,-1), 0, wxDefaultValidator, _T("ID_TEXTCTRL1"));
	FlexGridSizer4->Add(neighborThreshold, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	FlexGridSizer2->Add(FlexGridSizer4, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	FlexGridSizer5 = new wxFlexGridSizer(1, 2, 0, 0);
	wxString __wxRadioBoxChoices_1[2] = 
	{
		_("Whole Mesh"),
		_("Selected Elements")
	};
	DomainRadioBox = new wxRadioBox(this, ID_RADIOBOX1, _("Select Domain"), wxDefaultPosition, wxDefaultSize, 2, __wxRadioBoxChoices_1, 1, 0, wxDefaultValidator, _T("ID_RADIOBOX1"));
	DomainRadioBox->SetSelection(0);
	FlexGridSizer5->Add(DomainRadioBox, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	SelectButton = new wxButton(this, ID_BUTTON3, _("Select"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_BUTTON3"));
	SelectButton->Disable();
	FlexGridSizer5->Add(SelectButton, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	FlexGridSizer2->Add(FlexGridSizer5, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	FlexGridSizer1->Add(FlexGridSizer2, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	FlexGridSizer3 = new wxFlexGridSizer(1, 2, 0, 0);
	OKButton = new wxButton(this, ID_BUTTON1, _("OK"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_BUTTON1"));
	FlexGridSizer3->Add(OKButton, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	CancelButton = new wxButton(this, ID_BUTTON2, _("Cancel"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_BUTTON2"));
	FlexGridSizer3->Add(CancelButton, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	FlexGridSizer1->Add(FlexGridSizer3, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	SetSizer(FlexGridSizer1);
	FlexGridSizer1->Fit(this);
	FlexGridSizer1->SetSizeHints(this);
	
	Connect(ID_RADIOBOX1,wxEVT_COMMAND_RADIOBOX_SELECTED,(wxObjectEventFunction)&DGenerateNeighbors::OnDomainRadioBoxSelect);
	Connect(ID_BUTTON3,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&DGenerateNeighbors::OnSelectButtonClick);
	Connect(ID_BUTTON1,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&DGenerateNeighbors::OnOKButtonClick);
	Connect(ID_BUTTON2,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&DGenerateNeighbors::OnCancelButtonClick);
	//*)

    // fill GUI data into window
    neighborThreshold->WriteText(wxString::Format("%f", m_pGUIData->neighThreshold));
}

DGenerateNeighbors::~DGenerateNeighbors()
{
	//(*Destroy(DGenerateNeighbors)
	//*)
}

void DGenerateNeighbors::OnOKButtonClick(wxCommandEvent& event)
{
    // extract neighbor threshold angle
    wxString number = neighborThreshold->GetValue();
    double dNeighThreshold;
    number.ToDouble(&dNeighThreshold);
    if (dNeighThreshold<=0.0 || dNeighThreshold>=180.0){
        showErrorMessage("Neighbor threshold value should be greater than 0 and less than 180");
        return;
    }
    // get opposite angle (angle between normals)
    dNeighThreshold = 180.0-dNeighThreshold;
    // transform to radians
    dNeighThreshold *= pi/180;

    m_modified = true;

    // run neighbors operation mesh method
    if (DomainRadioBox->GetStringSelection() == _("Whole Mesh"))
        m_pMesh->updateNeighboringInfo(dNeighThreshold);
    else
        m_pMesh->updateNeighboringInfo(dNeighThreshold, m_elemSelection);

    this->Destroy();
}

void DGenerateNeighbors::OnCancelButtonClick(wxCommandEvent& event)
{
    this->Destroy();
}

void DGenerateNeighbors::OnDomainRadioBoxSelect(wxCommandEvent& event)
{
    // enable/disable select button
    if (DomainRadioBox->GetStringSelection() == _("Whole Mesh"))
        SelectButton->Enable(false);
    else
        SelectButton->Enable(true);
}

void DGenerateNeighbors::OnSelectButtonClick(wxCommandEvent& event)
{
    DElemSelection* ElemSelectionFrame = new DElemSelection(0, m_pMesh, m_elemSelection);
    ElemSelectionFrame->ShowModal();
}
