#include "wx_pch.h"
#include "DRun.h"

#ifndef WX_PRECOMP
//(*InternalHeadersPCH(DRun)
#include <wx/intl.h>
#include <wx/string.h>
//*)
#endif
//(*InternalHeaders(DRun)
#include <wx/font.h>
//*)

// control ids
enum {GaugePage_Timer};

//(*IdInit(DRun)
const long DRun::ID_TEXTCTRL1 = wxNewId();
const long DRun::ID_STATICTEXT2 = wxNewId();
const long DRun::ID_GAUGE1 = wxNewId();
const long DRun::ID_BUTTON1 = wxNewId();
const long DRun::ID_BUTTON3 = wxNewId();
const long DRun::ID_BUTTON2 = wxNewId();
//*)

BEGIN_EVENT_TABLE(DRun,wxDialog)
    //(*EventTable(DRun)
    //*)
    EVT_TIMER(GaugePage_Timer, DRun::OnProgressTimer)
END_EVENT_TABLE()

DRun::DRun(
        wxWindow* parent,
        bool& modified,
        CGUIData* pGUIData,
        CMesh* pMesh,
        CProjectData* pProjectData,
        CResults* pResults,
        wxWindowID id,
        const wxPoint& pos,
        const wxSize& size) : m_modified(modified), m_pGUIData(pGUIData), m_pMesh(pMesh), m_pProjectData(pProjectData), m_pResults(pResults)
{
    //(*Initialize(DRun)
    wxFlexGridSizer* FlexGridSizer2;
    wxFlexGridSizer* FlexGridSizer1;

    Create(parent, wxID_ANY, _("Run"), wxDefaultPosition, wxDefaultSize, wxSTAY_ON_TOP|wxCAPTION, _T("wxID_ANY"));
    FlexGridSizer1 = new wxFlexGridSizer(4, 1, 0, 0);
    FlexGridSizer1->AddGrowableCol(0);
    FlexGridSizer1->AddGrowableRow(0);
    logFileText = new wxTextCtrl(this, ID_TEXTCTRL1, wxEmptyString, wxDefaultPosition, wxSize(600,400), wxTE_MULTILINE, wxDefaultValidator, _T("ID_TEXTCTRL1"));
    wxFont logFileTextFont(9,wxSWISS,wxFONTSTYLE_NORMAL,wxNORMAL,false,_T("Courier New"),wxFONTENCODING_DEFAULT);
    logFileText->SetFont(logFileTextFont);
    FlexGridSizer1->Add(logFileText, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    state = new wxStaticText(this, ID_STATICTEXT2, _("Press \"Run\" to start Apame solver, or \"Cancel\" to abort..."), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT2"));
    FlexGridSizer1->Add(state, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 10);
    progressBar = new wxGauge(this, ID_GAUGE1, 100, wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_GAUGE1"));
    FlexGridSizer1->Add(progressBar, 1, wxALL|wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    FlexGridSizer2 = new wxFlexGridSizer(0, 3, 0, 0);
    RunButton = new wxButton(this, ID_BUTTON1, _("Run"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_BUTTON1"));
    FlexGridSizer2->Add(RunButton, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    ResultsButton = new wxButton(this, ID_BUTTON3, _("Load Results"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_BUTTON3"));
    ResultsButton->Disable();
    FlexGridSizer2->Add(ResultsButton, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    CancelCloseButton = new wxButton(this, ID_BUTTON2, _("Cancel"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_BUTTON2"));
    FlexGridSizer2->Add(CancelCloseButton, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    FlexGridSizer1->Add(FlexGridSizer2, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    SetSizer(FlexGridSizer1);
    FlexGridSizer1->Fit(this);
    FlexGridSizer1->SetSizeHints(this);

    Connect(ID_BUTTON1,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&DRun::OnRunButtonClick);
    Connect(ID_BUTTON3,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&DRun::OnResultsButtonClick);
    Connect(ID_BUTTON2,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&DRun::OnCancelCloseButtonClick);
    //*)

    // assemble ApameSolver jobname path
    if (m_pGUIData->useTempDir) m_jobNamePath = m_pGUIData->tempDir+"/";
    else                        m_jobNamePath = m_pGUIData->currentDir+"/";
    m_jobNamePath += replaceStringSegment(change2SingleSpace(m_pProjectData->projectName), " ", "_");
    
    // delete old input file
    if (deleteFile(m_jobNamePath+".inp"))
        this->Destroy();

    // delete old log file
    if (deleteFile(m_jobNamePath+".log"))
        this->Destroy();

    // delete old results file
    if (deleteFile(m_jobNamePath+".res"))
        this->Destroy();

    // initializing timer for updating log window
    m_timer = (wxTimer*)NULL;

    state->SetLabel("Press Run to start ApameSolver!");
}

DRun::~DRun()
{
    //(*Destroy(DRun)
    //*)
    delete m_timer;
}

void DRun::OnCancelCloseButtonClick(wxCommandEvent& event)
{
    this->Destroy();
}

void DRun::OnRunButtonClick(wxCommandEvent& event)
{
    // disable run button
    RunButton->Disable();

    // write ApameSolver input file
    if (writeApameSolverInput(m_jobNamePath+".inp", m_pMesh, m_pProjectData)){
       RunButton->Enable();
       return;
    }

    // instantiate new timer and start it with refresh rate defined in GUI data structure
    m_timer = new wxTimer(this, GaugePage_Timer);
    m_timer->Start(m_pGUIData->logWinRefreshRate);

    // run ApameSolver
    m_apameSolverProcess = wxProcess::Open(str2Wxs(m_pGUIData->apameSolverPath+" "+m_jobNamePath));

    // check if process successfully started
    if (!m_apameSolverProcess){
        showErrorMessage("ApameSolver could not be started, please check settings and/or environment!");
        m_timer->Stop();
    }
}

void DRun::OnProgressTimer(wxTimerEvent& WXUNUSED(event))
{
    // show log file on screen
    ifstream logFile((m_jobNamePath+".log").c_str());
    if (logFile.is_open()){
        // read log file
        vecStr logLines;
        string line;
        while (getLine(logFile, line))
            logLines.push_back(line);
        logFile.close();
        
        // if newly read vector is larger then old vector then update the log window
        if (logLines.size() > m_logLines.size()){
            // append only missing lines
            for (uInt i=m_logLines.size(); i<logLines.size(); i++)
                logFileText->AppendText(str2Wxs(logLines[i]+"\n"));

            // store new lines into member vector
            m_logLines.resize(logLines.size());
            for (uInt i=m_logLines.size(); i<m_logLines.size(); i++)
                m_logLines[i] = logLines[i];
        }
    }

    // if process still exists
    if (m_apameSolverProcess && m_apameSolverProcess->Exists(m_apameSolverProcess->GetPid()))
        state->SetLabel("Running ApameSolver...");
    else{
        // stop timer, update message and enable "Load Results" button
        m_timer->Stop();
        state->SetLabel("ApameSolver finished, press \"Load Results\" if job successful!");
        ResultsButton->Enable();
    }
}

void DRun::OnResultsButtonClick(wxCommandEvent& event)
{
    // get results file name
    string resFileName = m_jobNamePath+".res";

    // get element IDs and types from mesh
    vecUInt elemIDsFromMesh;
    vecUInt elemTypes;
    if (m_pMesh->getElemIDsTypes(elemIDsFromMesh, elemTypes)){
        showErrorMessage("Problem reading element IDs and types from mesh");
        return;
    }

    // get IDs of body elements in solver order (non-wake first)
    vecUInt elemIDs;
    for (uInt elemIndex=0; elemIndex<elemIDsFromMesh.size(); elemIndex++){
        if (elemTypes[elemIndex]!=10 && elemTypes[elemIndex]!=11)
            elemIDs.push_back(elemIDsFromMesh[elemIndex]);
    }

    // following operations will change (result) data
    m_modified = true;

    // read ApameSolver results file lines into results data class
    delete m_pResults;
    m_pResults = new CResults();
    if (readApameSolverResults(
            m_pResults,
            resFileName,
            elemIDs))
        return;

    CancelCloseButton->SetLabel(_("Close"));
    state->SetLabel(_("Results loaded"));
    ResultsButton->Disable();
}
