#include "generalFunctions.h"

//-----------------------------------------------------------------------/
// converts string to integer
//-----------------------------------------------------------------------/
int str2Int(const string& number)
{
    return atoi(number.c_str());
}

//-----------------------------------------------------------------------/
// converts string to double
//-----------------------------------------------------------------------/
double str2Dbl(const string& number)
{
    return atof(number.c_str());
}

//-----------------------------------------------------------------------/
// converts integer to string
//-----------------------------------------------------------------------/
string int2Str(const int& number)
{
    stringstream ss;
    ss << number;
    return ss.str();
}

//-----------------------------------------------------------------------/
// converts double to string
//-----------------------------------------------------------------------/
string dbl2Str(const double& number)
{
    stringstream ss;
    ss << number;
    return ss.str();
}

//-----------------------------------------------------------------------/
// converts wxString to stl string
//-----------------------------------------------------------------------/
string wxs2Str(const wxString& wxsString)
{
    return string(wxsString.mb_str());
}

//-----------------------------------------------------------------------/
// converts stl string to wxString
//-----------------------------------------------------------------------/
wxString str2Wxs(const string& stlString)
{
    wxString outputString(stlString);
    return outputString;
}

//-----------------------------------------------------------------------/
// converts number to wxString with 2 digits (plus a space for a minus
// sign) for whole number and 2 digits for decimal part (case angles form)
//-----------------------------------------------------------------------/
wxString dbl2StrCase(const double& number)
{
    // declare return variable
    wxString outString;

    // if greater then output space return blanks
    if (number >= 100.0)
        outString = " **.**";
    else if (number <= -100.0)
        outString = "-**.**";
    else{
        // reduce to two digits
        outString = wxString::Format("%.2f", number);
        
        // add space in place of minus if number positive
        if (number >= 0.0)
            outString = " "+outString;
    }

    return outString;
}

//-----------------------------------------------------------------------/
// converts number to wxString in scientific form with 7 digits for
// decimal part (force form)
//-----------------------------------------------------------------------/
wxString dbl2StrForc(const double& number)
{
    // declare return variable
    wxString outString = wxString::Format("%.7e", number);

    // add space in place of minus if number positive
    if (number >= 0.0)
        outString = " "+outString;

    return outString;
}

//-----------------------------------------------------------------------/
// converts number to wxString with one digit (plus a space for a minus
// sign) for whole number and 4 digits for decimal part (coefficient form)
//-----------------------------------------------------------------------/
wxString dbl2StrCoeff(const double& number)
{
    // declare return variable
    wxString outString;

    // if greater then output space return blanks
    if (number >= 10.0)
        outString = " **.****";
    else if (number <= -10.0)
        outString = "-**.****";
    else{
        // reduce to four digits
        outString = wxString::Format("%.4f", number);
        
        // add space in place of minus if number positive
        if (number >= 0.0)
            outString = " "+outString;
    }

    return outString;
}

//-----------------------------------------------------------------------/
// converts string to upper case
//-----------------------------------------------------------------------/
string convert2UpperCase(const string& inputString)
{
    string outputString = inputString;

    for (uInt i=0; i<outputString.length(); i++)
        outputString[i] = char(toupper(outputString[i]));

    return outputString;
}

//-----------------------------------------------------------------------/
// changes multiple spaces to single spaces
//-----------------------------------------------------------------------/
string change2SingleSpace(const string& inputString)
{
    string outputString = inputString;
    size_t position = outputString.find("  ");
    while (position != string::npos){
        outputString.replace(position, 2, " ");
        position = outputString.find("  ");
    }

    return outputString;
}

//-----------------------------------------------------------------------/
// removes leading spaces from string
//-----------------------------------------------------------------------/
string removeLeadingSpaces(const string& inputString)
{
    // count number of leading spaces
    uInt numSpaces = 0;
    for (; numSpaces<inputString.size(); numSpaces++){
        if (inputString[numSpaces] != ' ')
            break;
    }

    // return stripped string
    return inputString.substr(numSpaces);
}

//-----------------------------------------------------------------------/
// splits string with character and returns segments in vector of strings
//-----------------------------------------------------------------------/
vecStr splitString(
        const string& inputString,
        const char& splitter)
{
    // declare vector of strings to be filled with splitted values
    vecStr segments;

    uInt lastPos = 0;
    for (uInt i=0; i<inputString.size(); i++){
        // check if splitting character
        if (inputString[i] == splitter){
            // check if empty segment
            if (i != lastPos)
                segments.push_back(inputString.substr(lastPos, i-lastPos));
            lastPos = i+1;
        }
    }
    // store rest of the string
    if (lastPos < inputString.size())
        segments.push_back(inputString.substr(lastPos));

    return segments;
}

//-----------------------------------------------------------------------/
// replaces string segment
//-----------------------------------------------------------------------/
string replaceStringSegment(
        const string& inputString,
        const string& oldSegment,
        const string& newSegment)
{
    string outputString = inputString;

    // find position of character to be replaced
    size_t position = outputString.find(oldSegment);
    // check if end of string reached
    while (position != string::npos){
        // replace character
        outputString.replace(position, 1, newSegment);
        // find new position
        position = outputString.find(oldSegment, position+1);
    }

    return outputString;
}

//-----------------------------------------------------------------------/
// checks if file exists
//-----------------------------------------------------------------------/
bool fileExists(const string& filename)
{
    // attempt to get the file attributes
    struct stat stFileInfo;
    if (stat(filename.c_str(), &stFileInfo) == 0)
        return true;
    else
        return false;
}

//-----------------------------------------------------------------------/
// delete file (if exists)
//-----------------------------------------------------------------------/
int deleteFile(const string& filename)
{
    if (fileExists(filename)){
        if (remove(filename.c_str())){
            showErrorMessage("Cannot delete existing file "+filename);
            return -1;
        }
    }

    return 0;
}

//-----------------------------------------------------------------------/
// reads line from file (end of line support for all platforms)
//-----------------------------------------------------------------------/
bool getLine(ifstream& file, string& line)
{
    if (!getline(file, line))
        return false;
    else{
        if (!line.empty() && line[line.size()-1]=='\r')
            line.resize(line.size()-1);
        return true;
    }
}

//-----------------------------------------------------------------------/
// calculates weight factors for inverse distance weighting method
// inverse distance weighting method originally programmed by Julien Chaussee
//-----------------------------------------------------------------------/
int weightFactors(
        vecVecDbl& nodeWeights,
        const vecVecUInt& elementNodes,
        const vecVecDbl& nodeCoords,
        const vecVecDbl& elemCenterCoords)
{
    // check if input data is of appropriate size
    if (elementNodes.size() != elemCenterCoords.size())
        return -1;

    // resize vector of weights to match number of elements
    nodeWeights.resize(elementNodes.size());
    for (uInt elemIndex=0; elemIndex<elementNodes.size(); elemIndex++){
        // resize weights of current element to match number of nodes this elements is made of
        nodeWeights[elemIndex].resize(elementNodes[elemIndex].size());

        // for each node of current element calculate its weight in respect to given element center
        for (uInt nodeIndex=0; nodeIndex<elementNodes[elemIndex].size(); nodeIndex++){
            // check if node exists
            if (elementNodes[elemIndex][nodeIndex] >= nodeCoords.size())
                return -2;

            // calculate squared node distance from given element center
            double squaredDistance = pow((nodeCoords[elementNodes[elemIndex][nodeIndex]][0]-elemCenterCoords[elemIndex][0]), 2) +
                                     pow((nodeCoords[elementNodes[elemIndex][nodeIndex]][1]-elemCenterCoords[elemIndex][1]), 2) +
                                     pow((nodeCoords[elementNodes[elemIndex][nodeIndex]][2]-elemCenterCoords[elemIndex][2]), 2) ;

            // calculate inverse distance
            nodeWeights[elemIndex][nodeIndex] = 1/squaredDistance;
        }
    }

    return 0;
}

//-----------------------------------------------------------------------/
// calculates data at nodes by inverse distance weighting method
// inverse distance weighting method originally programmed by Julien Chaussee
//-----------------------------------------------------------------------/
int weightAverage(
        vecDbl& nodeField,
        const vecVecDbl& weights,
        const vecVecUInt& elementNodes,
        const vecDbl& elemField)
{
    // check sizes
    if (weights.size() > elemField.size())
        return -1;
    else if (weights.size() != elementNodes.size())
        return -2;

    // declare and allocate node sum
    vecDbl nodeSum(nodeField.size(), 0.0);

    // for each element
    for (uInt elemIndex=0; elemIndex<elementNodes.size(); elemIndex++){
        // for each node of current element
        for (uInt nodeIndex=0; nodeIndex<elementNodes[elemIndex].size(); nodeIndex++){
            // check that referenced node exists
            if (elementNodes[elemIndex][nodeIndex] >= nodeField.size())
                return -3;

            // add influence of node
            nodeField[elementNodes[elemIndex][nodeIndex]] += elemField[elemIndex]*weights[elemIndex][nodeIndex];

            // add node weight
            nodeSum[elementNodes[elemIndex][nodeIndex]] += weights[elemIndex][nodeIndex];
        }
    }

    // divide summed influences with weight sum
    for (uInt nodeIndex=0; nodeIndex<nodeField.size(); nodeIndex++){
        if (nodeSum[nodeIndex] == 0.0)
            nodeField[nodeIndex] = 0.0;
        else
            nodeField[nodeIndex] /= nodeSum[nodeIndex];
    }

    return 0;
}

//-----------------------------------------------------------------------/
// calculates element surface area based on nodes coordinates
//-----------------------------------------------------------------------/
double elementSurface(vecVecDbl& coords)
{
    // check input data
    if (coords.size()!=3 && coords.size()!=4)
        return 0.0;
    for (uInt i=0; i<coords.size(); i++){
        if (coords[i].size() != 3)
            return 0.0;
    }

    // calculating diagonal vectors
    double a[3], b[3];
                             for (uInt i=0; i<3; i++) a[i] = coords[2][i]-coords[0][i];
    if (coords.size() == 4){ for (uInt i=0; i<3; i++) b[i] = coords[3][i]-coords[1][i]; }
    else                   { for (uInt i=0; i<3; i++) b[i] = coords[2][i]-coords[1][i]; }

    // calculating vector normal to diagonals (vector product)
    double normal[3];
    normal[0] = a[1]*b[2] - a[2]*b[1];
    normal[1] = a[2]*b[0] - a[0]*b[2];
    normal[2] = a[0]*b[1] - a[1]*b[0];

    // calculating modulus of that vector
    double modulus = sqrt(pow(normal[0], 2) +
                          pow(normal[1], 2) +
                          pow(normal[2], 2) );

    // element surface is exactly half of that modulus
    return modulus/2;
}
