#include "undoRedo.h"

//-----------------------------------------------------------------------/
// performs operations when undo button clicked
//-----------------------------------------------------------------------/
void undo(
        list<undoRedoStruct>& undoList,
        list<undoRedoStruct>& redoList,
        CProjectData* pProjectData,
        CMesh* pMesh,
        CResults* pResults)
{
    // there is no undo data available; this should not happen, but in any case
    if (undoList.empty())
        return;

    // get last undo data
    undoRedoStruct undoData = undoList.front();

    // restore undo data
    *pProjectData = undoData.undoGUIData;
    *pMesh = undoData.undoMesh;
    *pResults = undoData.undoResults;

    // move undo data from undo list to redo list
    redoList.splice(redoList.begin(), undoList, undoList.begin());
}

//-----------------------------------------------------------------------/
// performs operations when redo button clicked
//-----------------------------------------------------------------------/
void redo(
        list<undoRedoStruct>& undoList,
        list<undoRedoStruct>& redoList,
        CProjectData* pProjectData,
        CMesh* pMesh,
        CResults* pResults)
{
    // there is no redo data available; this should not happen, but in any case
    if (redoList.empty())
        return;

    // get first redo data
    undoRedoStruct redoData = redoList.front();

    // restore redo data
    *pProjectData = redoData.redoGUIData;
    *pMesh = redoData.redoMesh;
    *pResults = redoData.redoResults;

    // move redo data from redo list to undo list
    undoList.splice(undoList.begin(), redoList, redoList.begin());
}

//-----------------------------------------------------------------------/
// stores undo data before change
//-----------------------------------------------------------------------/
void storeUndoData(
        const string& command,
        undoRedoStruct& undoInfo,
        CMesh* pMesh,
        CProjectData* pProjectData,
        CResults* pResults)
{
    // store data to undo/redo structure
    undoInfo.command = command;
    undoInfo.undoMesh = *pMesh;
    undoInfo.undoGUIData = *pProjectData;
    undoInfo.undoResults = *pResults;
}

//-----------------------------------------------------------------------/
// stores redo data after change
//-----------------------------------------------------------------------/
void storeRedoData(
        undoRedoStruct& undoInfo,
        list<undoRedoStruct>& undoList,
        CMesh* pMesh,
        CProjectData* pProjectData,
        CResults* pResults)
{
    // create new undo/redo structure
    undoRedoStruct newUndoInfo;

    // add it to undo list
    undoList.push_front(newUndoInfo);

    // copy undo info
    undoList.front().command = undoInfo.command;
    undoList.front().undoGUIData = undoInfo.undoGUIData;
    undoList.front().undoMesh = undoInfo.undoMesh;
    undoList.front().undoResults = undoInfo.undoResults;

    // add redo info
    undoList.front().redoGUIData = *pProjectData;
    undoList.front().redoMesh = *pMesh;
    undoList.front().redoResults = *pResults;
}
