// Copyright (C) 2011  Daniel Filkovic

// This file is part of APAME.

// APAME is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// APAME is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with APAME.  If not, see <http://www.gnu.org/licenses/>.

// file 01_example_plot_simple_wing.sci

// This is an example script that assembles and plots mesh (simple wing)

clear
close
clc

getd('simple_wing/')
getd('general/')

funcprot(0)
stacksize('max')
pi=4.0*atan(1.0);

// initialize data structures
[apame_data]=initialize_apame_data();
[plot_data]=initialize_plot_data();

// enter "1" to plot only body elements
plot_data.body_only=1;

//==============================================================================
// calling function that creates mesh
//==============================================================================
// geometry data
apame_data.wing_span=5;                               // wing span, from left to right wing tip (whole wing)
wing_data=struct("croot",1,    ...                    // root chord length            [m]
                 "ctip",0.3,    ...                   // tip chord length             [m]
                 "tipAL",-3*pi/180,    ...            // tip angle of attack          [radians]
                 "lambda14",30*pi/180,    ...         // wing sweep (1/4 wing)        [radians]
                 "dihedral",5*pi/180,    ...          // dihedral                     [radians]
                 "r_foil","airfoils/66415_40.DAT", ...// root airfoil data file
                 "t_foil","airfoils/66015_40.DAT", ...// tip airfoil data file
                 "wake_rows",10,    ...               // number of wake rows
                 "wake_length",10,    ...             // length of wake (in span multiples)
                 "N_max",100,    ...                  // max. airfoil panel number (for matrix initialization)
                 "Mp",15);                            // number of panels on halfspan
// wake data
wake_rows=10;    // number of wake rows
wake_length=10;  // length of wake (in span multiples)
// calculate mesh
[nodes,elements,apame_data]=simple_wing(apame_data,wing_data,wake_rows,wake_length);
//==============================================================================

// plot data
plot_data_func(plot_data,apame_data,nodes,elements);
