// Copyright (C) 2011  Daniel Filkovic

// This file is part of APAME.

// APAME is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// APAME is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with APAME.  If not, see <http://www.gnu.org/licenses/>.

// file 02_example_plot_lofted_wing.sci

// This is an example script that assembles and plots mesh (lofted wing)

clear
close
clc

getd('lofted_wing/')
getd('general/')

funcprot(0)
stacksize('max')
pi = 4.0*atan(1.0);

// initialize data structures
[apame_data]=initialize_apame_data();
[plot_data]=initialize_plot_data();

// enter "1" to plot only body elements
plot_data.body_only=1;

//==============================================================================
// calling function that creates mesh
//==============================================================================
// geometry data
apame_data.wing_span=20.0;       // wing span, from left to right wing tip (whole wing)
// airfoil coordinate files
airfoils={'airfoils/66415_40.dat', ...
          'airfoils/66415_40.dat', ...
          'airfoils/66415_40.dat', ...
          'airfoils/66415_40.dat', ...
          'airfoils/66015_40.dat'};
// airfoil locations (distribution)
x_af=[2.0, 2.4, 3.1, 4.8, 5.6];
y_af=[0.0, 0.2, 0.4, 0.8, 1.0];
z_af=[0.0, 0.1, 0.3, 0.7, 0.7];
// airfoil chord lengths
crd_af=[8.0, 4.1, 2.4, 1.4, 0.9];
// airfoil angles of attack
aoa_af=[2, 4, 4, 0, 2]*pi/180;
// wake data
wake_rows=10;    // number of wake rows
wake_length=10;  // length of wake (in span multiples)
// mesh parameters
min_el=0.02;    // minimum element y length (relative to halfspan)
max_el=0.06;    // maximum element y length (relative to halfspan)
N_max=100;      // max. airfoil panel number (for matrix initialization)
M_max=100;      // max. halfspan panel number (for matrix initialization)
// calculate mesh
[nodes,elements,apame_data]=lofted_wing(apame_data,airfoils,wake_rows,wake_length,x_af,y_af,z_af,crd_af,aoa_af,min_el,max_el,N_max,M_max);
//==============================================================================

// plot data
plot_data_func(plot_data,apame_data,nodes,elements);
