// Copyright (C) 2010-2011  Daniel Filkovic

// This file is part of APAME.

// APAME is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// APAME is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with APAME.  If not, see <http://www.gnu.org/licenses/>.

// file export_abaqus_loads.sci

// This script exports pressures to Abaqus file format. Manometer pressure
// as well as geometry, is assumed already read from ApameSolver results
// file with reader.sci script

// desired filename
abaqus_file = 'abaqus_loads.inp';
// starting node ID
node_shift = 1000000;
// starting element ID
element_shift = 1000000;

// open file
input_file = file('open',abaqus_file,'unknown');

// write nodes
write(input_file,'*NODE, SYSTEM=R')
for i = 1:node_num
  write(input_file,i+node_shift,'(1x,I10,$)')
  write(input_file,',','(a1,$)')
  write(input_file,node_x(i),'(3x,E14.8,$)')
  write(input_file,',','(a1,$)')
  write(input_file,node_y(i),'(3x,E14.8,$)')
  write(input_file,',','(a1,$)')
  write(input_file,node_z(i),'(3x,E14.8)')
end

// write elements
write(input_file,'*ELEMENT, TYPE=S4, ELSET=SKIN')
for i = 1:panel_num
  if panel_type(i) == 1
    write(input_file,i+element_shift,'(1x,I10,$)')
    write(input_file,',','(a1,$)')
    write(input_file,node_1(i)+node_shift,'(3x,I10,$)')
    write(input_file,',','(a1,$)')
    write(input_file,node_2(i)+node_shift,'(3x,I10,$)')
    write(input_file,',','(a1,$)')
    write(input_file,node_3(i)+node_shift,'(3x,I10,$)')
    write(input_file,',','(a1,$)')
    write(input_file,node_4(i)+node_shift,'(3x,I10)')
  end
end
write(input_file,'*ELEMENT, TYPE=S3, ELSET=SKIN')
for i = 1:panel_num
  if panel_type(i) == 2
    write(input_file,i+element_shift,'(1x,I10,$)')
    write(input_file,',','(a1,$)')
    write(input_file,node_1(i)+node_shift,'(3x,I10,$)')
    write(input_file,',','(a1,$)')
    write(input_file,node_2(i)+node_shift,'(3x,I10,$)')
    write(input_file,',','(a1,$)')
    write(input_file,node_3(i)+node_shift,'(3x,I10)')
  end
end

// write property data
write(input_file,'*SHELL SECTION, ELSET=SKIN, MATERIAL=STEEL')
write(input_file,'        1.,          ')

// write material data
write(input_file,'*MATERIAL, NAME=STEEL')
write(input_file,'*ELASTIC, TYPE=ISOTROPIC')
write(input_file,'   210000.,       0.3,        0.')
write(input_file,'*DENSITY')
write(input_file,'    7.8E-9')

// write step data
write(input_file,'*STEP, INC=100')
write(input_file,'Static_and_body_load')
write(input_file,'*STATIC')
write(input_file,'*NODE PRINT, FREQUENCY=1')
write(input_file,'    U,')
write(input_file,'*FILE FORMAT, ASCII')
write(input_file,'*EL PRINT, ELSET=SKIN,FREQUENCY=1, POSITION=CENTROIDAL')
write(input_file,'    S,')
write(input_file,' SINV,')

// write elemental load
write(input_file,'*DLOAD, OP=NEW')
for i = 1:panel_num
  if panel_type(i) == 1 | panel_type(i) == 2
    write(input_file,i+element_shift,'(1x,I10,$)')
    write(input_file,',    P,    ','(a12,$)')
    write(input_file,-p_mano(i),'(E14.8)')
  end
end

// finish file
write(input_file,'*END STEP')
file('close',input_file)
