// Copyright (C) 2011  Daniel Filkovic

// This file is part of APAME.

// APAME is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// APAME is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with APAME.  If not, see <http://www.gnu.org/licenses/>.

// file calculate_reference_data.sci

// This functions calculates reference surface, wetted surface and mean
// aerodynamic chord

function [apame_data] = calculate_reference_data(apame_data,nodes,elements,N,M)

// calculating surfaces
apame_data.wing_surf=0;
apame_data.S_wet=0;
for i=1:apame_data.elem_num
    // determining diagonals as vectors
    A=[nodes(elements(i,4),1)-nodes(elements(i,2),1), nodes(elements(i,4),2)-nodes(elements(i,2),2), nodes(elements(i,4),3)-nodes(elements(i,2),3)];
    B=[nodes(elements(i,5),1)-nodes(elements(i,3),1), nodes(elements(i,5),2)-nodes(elements(i,3),2), nodes(elements(i,5),3)-nodes(elements(i,3),3)];

    // calculating vector normal to diagonals (vector product)
    C=zeros(1,3);
    C(1) = A(2)*B(3)-A(3)*B(2);
    C(2) = A(3)*B(1)-A(1)*B(3);
    C(3) = A(1)*B(2)-A(2)*B(1);
    
    // calculating modulus of that vector
    C_mod=sqrt(C(1)^2 + C(2)^2 + C(3)^2);
    
    // panel surface is exactly half of that modulus
    if C_mod>0 then
        S=C_mod/2;
        apame_data.S_wet=apame_data.S_wet+S;
        
        // adding to reference surface
        if C(3)>0 then
            apame_data.wing_surf=apame_data.wing_surf+C(3)/2;
        end
    end
end

// calculating mean aerodynamic chord
apame_data.mac=0;
for i=1:M-1
    j=round(N/2);    // leading edge node index
    dist_sq=(x(1,i)-x(j,i))^2 + (z(1,i)-z(j,i))^2;
    apame_data.mac=apame_data.mac+dist_sq*(y(1,i+1)-y(1,i));
end
apame_data.mac=apame_data.mac*2/apame_data.wing_surf;

endfunction
