// Copyright (C) 2011  Daniel Filkovic

// This file is part of APAME.

// APAME is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// APAME is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with APAME.  If not, see <http://www.gnu.org/licenses/>.

// file convert_to_unstructured.sci

// This functions converts structured mesh (x,y,z) to unstructured
// (nodes,elements) and adds mirror image ZX simmetry

function [apame_data,nodes,elements] = convert_to_unstructured(apame_data,N,M,x,y,z)

apame_data.elem_num=(N-1)*(M-1)*2;
apame_data.node_num=N*M*2;

// declare node coordinates matrix and fill with data
nodes=zeros(apame_data.node_num,3);
count=0;
// left wing
for j=1:M
    for i=1:N
        count=count+1;
        nodes(count,1)=x(i,j);
        nodes(count,2)=-y(i,j);
        nodes(count,3)=z(i,j);
    end
end
// right wing
for j=1:M
    for i=1:N
        count=count+1;
        nodes(count,1)=x(i,j);
        nodes(count,2)=y(i,j);
        nodes(count,3)=z(i,j);
    end
end

// declare elements definition matrix and fill with data
elements=zeros(apame_data.elem_num,9);
count=0;
// left wing
for j=1:M-1
    for i=1:N-1
        count=count+1;
        // element type
        elements(count,1)=1;
        // constituing nodes
        elements(count,2)=N*(j-1)+i;
        elements(count,3)=N*j+i;
        elements(count,4)=N*j+i+1;
        elements(count,5)=N*(j-1)+i+1;
        
        // neighbors data
        if i == 1 then
            if j == 1 then
                // lower left corner element
                elements(count,6)=2;
                elements(count,7)=(N-1)+2;
                elements(count,8)=(N-1)+1;
            elseif j == M-1 then
                // lower right corner element
                elements(count,6)=(N-1)*(M-3)+1;
                elements(count,7)=(N-1)*(M-3)+2;
                elements(count,8)=(N-1)*(M-2)+2;
            else
                // lower side elements
                elements(count,6)=(N-1)*(j-2)+1;
                elements(count,7)=(N-1)*(j-1)+2;
                elements(count,8)=(N-1)*j+1;
            end
        elseif i == N-1 then
            if j == 1 then
                // upper left corner element
                elements(count,6)=N-2;
                elements(count,7)=2*(N-1)-1;
                elements(count,8)=2*(N-1);
            elseif j == M-1 then
                // upper right corner element
                elements(count,6)=(N-1)*(M-2);
                elements(count,7)=(N-1)*(M-2)-1;
                elements(count,8)=(N-1)*(M-1)-1;
            else
                // upper side elements
                elements(count,6)=(N-1)*(j-1);
                elements(count,7)=(N-1)*j-1;
                elements(count,8)=(N-1)*(j+1);
            end
        else
            if j == 1 then
                // left side elements
                elements(count,6)=i+1;
                elements(count,7)=(N-1)+i;
                elements(count,8)=i-1;
            elseif j == M-1 then
                // right side elements
                elements(count,6)=(N-1)*(M-2)+i-1;
                elements(count,7)=(N-1)*(M-3)+i;
                elements(count,8)=(N-1)*(M-2)+i+1;
            else
                // inner elements (with full info)
                elements(count,6)=(N-1)*(j-2)+i;
                elements(count,7)=(N-1)*(j-1)+i+1;
                elements(count,8)=(N-1)*j+i;
                elements(count,9)=(N-1)*(j-1)+i-1;
            end
        end
    end
end
// right wing
for j=1:M-1
    for i=1:N-1
        count=count+1;
        // element type
        elements(count,1)=1;
        // constituing nodes
        elements(count,2)=N*(j-1)+i+apame_data.node_num/2;    // first
        elements(count,3)=N*(j-1)+i+1+apame_data.node_num/2;  // second
        elements(count,4)=N*j+i+1+apame_data.node_num/2;      // third
        elements(count,5)=N*j+i+apame_data.node_num/2;        // fourth
        
        // neighbors data
        if i == 1 then
            if j == 1 then
                // lower left corner element
                elements(count,6)=apame_data.elem_num/2+2;
                elements(count,7)=apame_data.elem_num/2+(N-1)+2;
                elements(count,8)=apame_data.elem_num/2+(N-1)+1;
            elseif j == M-1 then
                // lower right corner element
                elements(count,6)=apame_data.elem_num/2+(N-1)*(M-3)+1;
                elements(count,7)=apame_data.elem_num/2+(N-1)*(M-3)+2;
                elements(count,8)=apame_data.elem_num/2+(N-1)*(M-2)+2;
            else
                // lower side elements
                elements(count,6)=apame_data.elem_num/2+(N-1)*(j-2)+1;
                elements(count,7)=apame_data.elem_num/2+(N-1)*(j-1)+2;
                elements(count,8)=apame_data.elem_num/2+(N-1)*j+1;
            end
        elseif i == N-1 then
            if j == 1 then
                // upper left corner element
                elements(count,6)=apame_data.elem_num/2+N-2;
                elements(count,7)=apame_data.elem_num/2+2*(N-1)-1;
                elements(count,8)=apame_data.elem_num/2+2*(N-1);
            elseif j == M-1 then
                // upper right corner element
                elements(count,6)=apame_data.elem_num/2+(N-1)*(M-2);
                elements(count,7)=apame_data.elem_num/2+(N-1)*(M-2)-1;
                elements(count,8)=apame_data.elem_num/2+(N-1)*(M-1)-1;
            else
                // upper side elements
                elements(count,6)=apame_data.elem_num/2+(N-1)*(j-1);
                elements(count,7)=apame_data.elem_num/2+(N-1)*j-1;
                elements(count,8)=apame_data.elem_num/2+(N-1)*(j+1);
            end
        else
            if j == 1 then
                // left side elements
                elements(count,6)=apame_data.elem_num/2+i+1;
                elements(count,7)=apame_data.elem_num/2+(N-1)+i;
                elements(count,8)=apame_data.elem_num/2+i-1;
            elseif j == M-1 then
                // right side elements
                elements(count,6)=apame_data.elem_num/2+(N-1)*(M-2)+i-1;
                elements(count,7)=apame_data.elem_num/2+(N-1)*(M-3)+i;
                elements(count,8)=apame_data.elem_num/2+(N-1)*(M-2)+i+1;
            else
                // inner elements (with full info)
                elements(count,6)=apame_data.elem_num/2+(N-1)*(j-2)+i;    // first
                elements(count,7)=apame_data.elem_num/2+(N-1)*(j-1)+i+1;  // second
                elements(count,8)=apame_data.elem_num/2+(N-1)*j+i;        // third
                elements(count,9)=apame_data.elem_num/2+(N-1)*(j-1)+i-1;  // fourth
            end
        end
    end
end

endfunction
