// Copyright (C) 2011  Daniel Filkovic

// This file is part of APAME.

// APAME is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// APAME is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with APAME.  If not, see <http://www.gnu.org/licenses/>.

// file create_wake.sci

// This functions creates wake elements to an existing structured mesh

function [nodes,elements,apame_data] = create_wake(nodes,elements,N,M,x,y,z,wake_rows,wake_length,apame_data)

node_count=0;
elem_count=0;
// left wing
for i=1:M-1
    // for each trailing pair
    for j=1:wake_rows
        // for each row of wake elements
        node_count=node_count+1;
        nodes(apame_data.node_num+node_count,1)=x(1,i)+wake_length*(j-1)/wake_rows*apame_data.wing_span;
        nodes(apame_data.node_num+node_count,2)=-y(1,i);
        nodes(apame_data.node_num+node_count,3)=z(1,i);
        node_count=node_count+1;
        nodes(apame_data.node_num+node_count,1)=x(1,i+1)+wake_length*(j-1)/wake_rows*apame_data.wing_span;
        nodes(apame_data.node_num+node_count,2)=-y(1,i+1);
        nodes(apame_data.node_num+node_count,3)=z(1,i+1);
        node_count=node_count+1;
        nodes(apame_data.node_num+node_count,1)=x(1,i+1)+wake_length*j/wake_rows*apame_data.wing_span;
        nodes(apame_data.node_num+node_count,2)=-y(1,i+1);
        nodes(apame_data.node_num+node_count,3)=z(1,i+1);
        node_count=node_count+1;
        nodes(apame_data.node_num+node_count,1)=x(1,i)+wake_length*j/wake_rows*apame_data.wing_span;
        nodes(apame_data.node_num+node_count,2)=-y(1,i);
        nodes(apame_data.node_num+node_count,3)=z(1,i);

        elem_count=elem_count+1;
        elements(apame_data.elem_num+elem_count,1)=10;
        elements(apame_data.elem_num+elem_count,2)=apame_data.node_num+node_count-3;
        elements(apame_data.elem_num+elem_count,3)=apame_data.node_num+node_count-2;
        elements(apame_data.elem_num+elem_count,4)=apame_data.node_num+node_count-1;
        elements(apame_data.elem_num+elem_count,5)=apame_data.node_num+node_count;
        elements(apame_data.elem_num+elem_count,6)=(i-1)*(N-1)+N-1;
        elements(apame_data.elem_num+elem_count,7)=(i-1)*(N-1)+1;
    end
end
// right wing
for i=1:M-1
    // for each trailing pair
    for j=1:wake_rows
        // for each row of wake elements
        node_count=node_count+1;
        nodes(apame_data.node_num+node_count,1)=x(1,i)+wake_length*(j-1)/wake_rows*apame_data.wing_span;
        nodes(apame_data.node_num+node_count,2)=y(1,i);
        nodes(apame_data.node_num+node_count,3)=z(1,i);
        node_count=node_count+1;
        nodes(apame_data.node_num+node_count,1)=x(1,i+1)+wake_length*(j-1)/wake_rows*apame_data.wing_span;
        nodes(apame_data.node_num+node_count,2)=y(1,i+1);
        nodes(apame_data.node_num+node_count,3)=z(1,i+1);
        node_count=node_count+1;
        nodes(apame_data.node_num+node_count,1)=x(1,i+1)+wake_length*j/wake_rows*apame_data.wing_span;
        nodes(apame_data.node_num+node_count,2)=y(1,i+1);
        nodes(apame_data.node_num+node_count,3)=z(1,i+1);
        node_count=node_count+1;
        nodes(apame_data.node_num+node_count,1)=x(1,i)+wake_length*j/wake_rows*apame_data.wing_span;
        nodes(apame_data.node_num+node_count,2)=y(1,i);
        nodes(apame_data.node_num+node_count,3)=z(1,i);

        elem_count=elem_count+1;
        elements(apame_data.elem_num+elem_count,1)=10;
        elements(apame_data.elem_num+elem_count,2)=apame_data.node_num+node_count-3;
        elements(apame_data.elem_num+elem_count,3)=apame_data.node_num+node_count;
        elements(apame_data.elem_num+elem_count,4)=apame_data.node_num+node_count-1;
        elements(apame_data.elem_num+elem_count,5)=apame_data.node_num+node_count-2;
        elements(apame_data.elem_num+elem_count,6)=apame_data.elem_num/2+(i-1)*(N-1)+N-1;
        elements(apame_data.elem_num+elem_count,7)=apame_data.elem_num/2+(i-1)*(N-1)+1;
    end
end

apame_data.node_num=size(nodes,"r");
apame_data.elem_num=size(elements,"r");

endfunction
