// Copyright (C) 2011  Daniel Filkovic

// This file is part of APAME.

// APAME is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// APAME is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with APAME.  If not, see <http://www.gnu.org/licenses/>.

// file plot_data_func.sci

// This function plots data

function plot_data_func(plot_data,apame_data,nodes,elements)

// check if field data will be plot
if plot_data.plot_field==1 then
    // select which data to plot
    if size(plot_data.field_data,"c")==1 then
        field=plot_data.field_data;
    else
        field=plot_data.field_data(:,plot_data.plot_case);
    end
    
    // crop field data
    if plot_data.crop==1 then
        for i=1:length(field)
            if field(i) < plot_data.crop_min
                field(i) = plot_data.crop_min;
            elseif field(i) > plot_data.crop_max
                field(i) = plot_data.crop_max;
            end
        end
    end
    
    // scale and translate data to plot domain
    field = field - min(field);
    field = field/max(field)*511 + 1;
    
    // calculate plot data in nodes
    if plot_data.smooth_plot==1 then
        field_node=zeros(apame_data.node_num,1);
        node_sum=zeros(apame_data.node_num,1);
        for i=1:apame_data.elem_num
            if elements(i,1) == 1
                field_node(elements(i,2)) = field_node(elements(i,2)) + field(i);
                field_node(elements(i,3)) = field_node(elements(i,3)) + field(i);
                field_node(elements(i,4)) = field_node(elements(i,4)) + field(i);
                field_node(elements(i,5)) = field_node(elements(i,5)) + field(i);
                node_sum(elements(i,2)) = node_sum(elements(i,2)) + 1;
                node_sum(elements(i,3)) = node_sum(elements(i,3)) + 1;
                node_sum(elements(i,4)) = node_sum(elements(i,4)) + 1;
                node_sum(elements(i,5)) = node_sum(elements(i,5)) + 1;
            elseif elements(i,1) == 2
                field_node(elements(i,2)) = field_node(elements(i,2)) + field(i);
                field_node(elements(i,3)) = field_node(elements(i,3)) + field(i);
                field_node(elements(i,4)) = field_node(elements(i,4)) + field(i);
                node_sum(elements(i,2)) = node_sum(elements(i,2)) + 1;
                node_sum(elements(i,3)) = node_sum(elements(i,3)) + 1;
                node_sum(elements(i,4)) = node_sum(elements(i,4)) + 1;
            end
        end
        
        // average the data
        for i=1:apame_data.node_num
            if node_sum(i) ~= 0 then
                field_node(i) = field_node(i) / node_sum(i);
            end
        end
    end
end

// count panel types
count_1=0;
count_2=0;
count_20=0;
count_21=0;
count_10=0;
count_11=0;
for i=1:apame_data.elem_num
    if elements(i,1) == 1
        count_1=count_1 + 1;
    elseif elements(i,1) == 2
        count_2=count_2 + 1;
    elseif elements(i,1) == 20
        count_20=count_20 + 1;
    elseif elements(i,1) == 21
        count_21=count_21 + 1;
    elseif elements(i,1) == 10
        count_10=count_10 + 1;
    elseif elements(i,1) == 11
        count_11=count_11 + 1;
    end
end

// assemble plot data
j_1=0;
j_2=0;
j_10=0;
j_11=0;
j_20=0;
j_21=0;
X_1 = zeros(4,count_1);
Y_1 = zeros(4,count_1);
Z_1 = zeros(4,count_1);
X_2 = zeros(3,count_2);
Y_2 = zeros(3,count_2);
Z_2 = zeros(3,count_2);
X_20 = zeros(4,count_20);
Y_20 = zeros(4,count_20);
Z_20 = zeros(4,count_20);
X_21 = zeros(3,count_21);
Y_21 = zeros(3,count_21);
Z_21 = zeros(3,count_21);
X_10 = zeros(4,count_10);
Y_10 = zeros(4,count_10);
Z_10 = zeros(4,count_10);
X_11 = zeros(3,count_11);
Y_11 = zeros(3,count_11);
Z_11 = zeros(3,count_11);
if plot_data.smooth_plot==1 then
    field_1 = zeros(4,count_1);
    field_2 = zeros(3,count_2);
else
    field_1 = zeros(count_1);
    field_2 = zeros(count_2);
end
for i=1:apame_data.elem_num
    if elements(i,1) == 1
        j_1=j_1 + 1;
        X_1(:,j_1)=[nodes(elements(i,2),1);nodes(elements(i,5),1);nodes(elements(i,4),1);nodes(elements(i,3),1)];
        Y_1(:,j_1)=[nodes(elements(i,2),2);nodes(elements(i,5),2);nodes(elements(i,4),2);nodes(elements(i,3),2)];
        Z_1(:,j_1)=[nodes(elements(i,2),3);nodes(elements(i,5),3);nodes(elements(i,4),3);nodes(elements(i,3),3)];        
        if plot_data.plot_field==1 then
            if plot_data.smooth_plot==1
                field_1(:,j_1) = [field_node(elements(i,2));field_node(elements(i,5));field_node(elements(i,4));field_node(elements(i,3))];
            else
                field_1(j_1) = field(i);
            end
        end
    elseif elements(i,1) == 2
        j_2=j_2 + 1;
        X_2(:,j_2)=[nodes(elements(i,2),1);nodes(elements(i,4),1);nodes(elements(i,3),1)];
        Y_2(:,j_2)=[nodes(elements(i,2),2);nodes(elements(i,4),2);nodes(elements(i,3),2)];
        Z_2(:,j_2)=[nodes(elements(i,2),3);nodes(elements(i,4),3);nodes(elements(i,3),3)];
        if plot_data.plot_field==1 then
            if plot_data.smooth_plot==1
                field_2(:,j_2) = [field_node(elements(i,2));field_node(elements(i,4));field_node(elements(i,3))];
            else
                field_2(j_2) = field(i);
            end
        end
    elseif elements(i,1) == 20
        j_20=j_20 + 1;
        X_20(:,j_20)=[nodes(elements(i,2),1);nodes(elements(i,5),1);nodes(elements(i,4),1);nodes(elements(i,3),1)];
        Y_20(:,j_20)=[nodes(elements(i,2),2);nodes(elements(i,5),2);nodes(elements(i,4),2);nodes(elements(i,3),2)];
        Z_20(:,j_20)=[nodes(elements(i,2),3);nodes(elements(i,5),3);nodes(elements(i,4),3);nodes(elements(i,3),3)];
    elseif elements(i,1) == 21
        j_21=j_21 + 1;
        X_21(:,j_21)=[nodes(elements(i,2),1);nodes(elements(i,4),1);nodes(elements(i,3),1)];
        Y_21(:,j_21)=[nodes(elements(i,2),2);nodes(elements(i,4),2);nodes(elements(i,3),2)];
        Z_21(:,j_21)=[nodes(elements(i,2),3);nodes(elements(i,4),3);nodes(elements(i,3),3)];
    elseif elements(i,1) == 10
        j_10=j_10 + 1;
        X_10(:,j_10)=[nodes(elements(i,2),1);nodes(elements(i,5),1);nodes(elements(i,4),1);nodes(elements(i,3),1)];
        Y_10(:,j_10)=[nodes(elements(i,2),2);nodes(elements(i,5),2);nodes(elements(i,4),2);nodes(elements(i,3),2)];
        Z_10(:,j_10)=[nodes(elements(i,2),3);nodes(elements(i,5),3);nodes(elements(i,4),3);nodes(elements(i,3),3)];
    elseif elements(i,1) == 11
        j_11=j_11 + 1;
        X_11(:,j_11)=[nodes(elements(i,2),1);nodes(elements(i,4),1);nodes(elements(i,3),1)];
        Y_11(:,j_11)=[nodes(elements(i,2),2);nodes(elements(i,4),2);nodes(elements(i,3),2)];
        Z_11(:,j_11)=[nodes(elements(i,2),3);nodes(elements(i,4),3);nodes(elements(i,3),3)];
    end
end

// plot data
if count_1 ~= 0
    if plot_data.plot_field==1 then
        plot3d(X_1,Y_1,list(Z_1,field_1))
        h = gce();
        h.color_mode=-2;    // remove boundary color
    else
        plot3d(X_1,Y_1,Z_1)
        h=gce();
        h.color_mode=plot_data.body_color;
    end
end
if count_2 ~= 0
    if plot_data.plot_field==1 then
        plot3d(X_2,Y_2,list(Z_2,field_2))
        h = gce();
        h.color_mode=-2;    // remove boundary color
    else
        plot3d(X_2,Y_2,Z_2)
        h=gce();
        h.color_mode=plot_data.body_color;
    end
end
if count_20 ~= 0
    plot3d(X_20,Y_20,Z_20)
    h=gce();
    h.color_mode=plot_data.dummy_color;
end
if count_21 ~= 0
    plot3d(X_21,Y_21,Z_21)
    h=gce();
    h.color_mode=plot_data.dummy_color;
end
if count_10 ~= 0 & plot_data.body_only ~= 1
    plot3d(X_10,Y_10,Z_10)
    h=gce();
    h.color_mode=plot_data.wake_color;
end
if count_11 ~= 0 & plot_data.body_only ~= 1
    plot3d(X_11,Y_11,Z_11)
    h=gce();
    h.color_mode=plot_data.wake_color;
end

// figure properties
if plot_data.plot_field==1 then
    f = gcf();
    f.color_map = jetcolormap(512);
end
a = gca();
a.box = "off";
a.isoview = "on";
a.cube_scaling = "off";
a.axes_visible="on";

endfunction
