// Copyright (C) 2010-2011  Daniel Filkovic

// This file is part of APAME.

// APAME is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// APAME is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with APAME.  If not, see <http://www.gnu.org/licenses/>.

// file read_airfoils.sci

// This function reads airfoil files given as arguments and outputs
// coordinate arrays and matrix size

function [x,y,N] = read_airfoils(airfoils,N_max,M_seg)

// initializing x and y coordinates
x = zeros(N_max,M_seg);
y = zeros(N_max,M_seg);
for i=1:M_seg
    a=fscanfMat(char(airfoils(i)));         // reading airfoil data
    red = size(a);                          // getting data size
    N = red(1); N1=N+1;                     // N=N1 in apame
    if N > N_max
        disp('ERROR: Maximum airfoil panel number reached, increase panel size!')
        break
    end
    amod = zeros(red);
    for j = 1:N
        amod(j,1)=a((N1-j),1);              // reversing data
        amod(j,2)=a((N1-j),2);
    end
    x(1:N,i) = (amod(:,1)-0.25);            // ZERO is quarter chord
    y(1:N,i) = (amod(:,2));
end
x = x(1:N,1:M_seg);
y = y(1:N,1:M_seg);

endfunction
