// Copyright (C) 2011  Daniel Filkovic

// This file is part of APAME.

// APAME is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// APAME is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with APAME.  If not, see <http://www.gnu.org/licenses/>.

// file read_apame_results.sci

// This function reads ApameSolver results file and stores data to
// apame_results_data structure

function apame_results_data=read_apame_results(file_name)

apame_results_data=struct("file_name",file_name);

// open file
results_file = file('open',file_name,'old');

// read file
ending = 0;
while ending == 0
  line = read(results_file,1,1,"(A)");
  if line == ' PANEL_NUM'
    panel_num = read(results_file,1,1);
    apame_results_data.panel_num=panel_num;
  elseif line == ' PANEL_NUM_NO_WAKE'
    panel_num_no_wake = read(results_file,1,1);
    apame_results_data.panel_num_no_wake=panel_num_no_wake;
  elseif line == ' NODE_NUM'
    node_num = read(results_file,1,1);
    apame_results_data.node_num=node_num;
  elseif line == ' NODES'
    temp = read(results_file,node_num,3);
    node_x = temp(:,1);
    node_y = temp(:,2);
    node_z = temp(:,3);
    apame_results_data.node_x=node_x;
    apame_results_data.node_y=node_y;
    apame_results_data.node_z=node_z;
  elseif line == ' PANELS'
    temp =  read(results_file,panel_num,9);
    panel_type = temp(:,1);
    node_1 = temp(:,2);
    node_2 = temp(:,3);
    node_3 = temp(:,4);
    node_4 = temp(:,5);
    elem_1 = temp(:,6);
    elem_2 = temp(:,7);
    elem_3 = temp(:,8);
    elem_4 = temp(:,9);
    apame_results_data.panel_type=panel_type;
    apame_results_data.node_1=node_1;
    apame_results_data.node_2=node_2;
    apame_results_data.node_3=node_3;
    apame_results_data.node_4=node_4;
    apame_results_data.elem_1=elem_1;
    apame_results_data.elem_2=elem_2;
    apame_results_data.elem_3=elem_3;
    apame_results_data.elem_4=elem_4;
  elseif line == ' AIRSPEED'
    airspeed = read(results_file,1,1);
    apame_results_data.airspeed=airspeed;
  elseif line == ' DENSITY'
    density = read(results_file,1,1);
    apame_results_data.density=density;
  elseif line == ' PRESSURE'
    pressure = read(results_file,1,1);
    apame_results_data.pressure=pressure;
  elseif line == ' MACH'
    mach = read(results_file,1,1);
    apame_results_data.mach=mach;
  elseif line == ' CASE_NUM'
    case_num = read(results_file,1,1);
    apame_results_data.case_num=case_num;
    alfa = read(results_file,1,case_num);
    apame_results_data.alfa=alfa;
    beta_ = read(results_file,1,case_num);
    apame_results_data.beta=beta_;
  elseif line == ' WINGSPAN'
    wingspan = read(results_file,1,1);
    apame_results_data.wingspan=wingspan;
  elseif line == ' MAC'
    mac = read(results_file,1,1);
    apame_results_data.mac=mac;
  elseif line == ' SURFACE'
    surface = read(results_file,1,1);
    apame_results_data.surface=surface;
  elseif line == ' ORIGIN'
    origin = read(results_file,1,3);
    apame_results_data.origin=origin;
  elseif line == ' METHOD'
    method = read(results_file,1,1);
    apame_results_data.method=method;
  elseif line == ' ERROR'
    err = read(results_file,1,1);
    apame_results_data.err=err;
  elseif line == ' COPL_ANG'
    copl_ang = read(results_file,1,1);
    copl_ang=copl_ang;
  elseif line == ' FARFIELD'
    farfield = read(results_file,1,1);
    apame_results_data.farfield=farfield;
  elseif line == ' COLLCALC'
    collcalc = read(results_file,1,1);
    apame_results_data.collcalc=collcalc;
  elseif line == ' VELORDER'
    velorder = read(results_file,1,1);
    apame_results_data.velorder=velorder;
  elseif line == ' CX'
    CX = read(results_file,1,case_num);
    apame_results_data.CX=CX;
  elseif line == ' CY'
    CY = read(results_file,1,case_num);
    apame_results_data.CY=CY;
  elseif line == ' CZ'
    CZ = read(results_file,1,case_num);
    apame_results_data.CZ=CZ;
  elseif line == ' CL'
    CL = read(results_file,1,case_num);
    apame_results_data.CL=CL;
  elseif line == ' CM'
    CM = read(results_file,1,case_num);
    apame_results_data.CM=CM;
  elseif line == ' CN'
    CN = read(results_file,1,case_num);
    apame_results_data.CN=CN;
  elseif line == ' CDRAG'
    CDRAG = read(results_file,1,case_num);
    apame_results_data.CDRAG=CDRAG;
  elseif line == ' CSIDE'
    CSIDE = read(results_file,1,case_num);
    apame_results_data.CSIDE=CSIDE;
  elseif line == ' CLIFT'
    CLIFT = read(results_file,1,case_num);
    apame_results_data.CLIFT=CLIFT;
  elseif line == ' FX'
    FX = read(results_file,1,case_num);
    apame_results_data.FX=FX;
  elseif line == ' FY'
    FY = read(results_file,1,case_num);
    apame_results_data.FY=FY;
  elseif line == ' FZ'
    FZ = read(results_file,1,case_num);
    apame_results_data.FZ=FZ;
  elseif line == ' FL'
    FL = read(results_file,1,case_num);
    apame_results_data.FL=FL;
  elseif line == ' FM'
    FM = read(results_file,1,case_num);
    apame_results_data.FM=FM;
  elseif line == ' FN'
    FN = read(results_file,1,case_num);
    apame_results_data.FN=FN;
  elseif line == ' FDRAG'
    FDRAG = read(results_file,1,case_num);
    apame_results_data.FDRAG=FDRAG;
  elseif line == ' FSIDE'
    FSIDE = read(results_file,1,case_num);
    apame_results_data.FSIDE=FSIDE;
  elseif line == ' FLIFT'
    FLIFT = read(results_file,1,case_num);
    apame_results_data.FLIFT=FLIFT;
  elseif line == ' VELOCITY'
    v = read(results_file,panel_num_no_wake,case_num);
    apame_results_data.v=v;
  elseif line == ' CP'
    cp = read(results_file,panel_num_no_wake,case_num);
    apame_results_data.cp=cp;
  elseif line == ' CENTER POINTS'
    temp = read(results_file,panel_num,3);
    cx = temp(:,1);
    cy = temp(:,2);
    cz = temp(:,3);
    apame_results_data.cx=cx;
    apame_results_data.cy=cy;
    apame_results_data.cz=cz;
  elseif line == ' DIPOLE'
    doublet = read(results_file,panel_num_no_wake,case_num);
    apame_results_data.doublet=doublet;
  elseif line == ' SOURCE'
    source = read(results_file,panel_num_no_wake,case_num);
    apame_results_data.source=source;
  elseif line == ' VELOCITY_COMP'
    for i=1:case_num
      temp = read(results_file,panel_num_no_wake,3);
      vx(:,i) = temp(:,1);
      vy(:,i) = temp(:,2);
      vz(:,i) = temp(:,3);
    end
    apame_results_data.vx=vx;
    apame_results_data.vy=vy;
    apame_results_data.vz=vz;
  elseif line == ' S'
    S =  read(results_file,panel_num,1);
    apame_results_data.S=S;
  elseif line == ' FF'
    FF = read(results_file,panel_num,1);
    apame_results_data.FF=FF;
  elseif line == ' NORMAL'
    temp = read(results_file,panel_num,3);
    nx = temp(:,1);
    ny = temp(:,2);
    nz = temp(:,3);
    apame_results_data.nx=nx;
    apame_results_data.ny=ny;
    apame_results_data.nz=nz;
  elseif line == ' L_VECTOR'
    temp = read(results_file,panel_num,3);
    lx = temp(:,1);
    ly = temp(:,2);
    lz = temp(:,3);
    apame_results_data.lx=lx;
    apame_results_data.ly=ly;
    apame_results_data.lz=lz;
  elseif line == ' P_VECTOR'
    temp = read(results_file,panel_num,3);
    px = temp(:,1);
    py = temp(:,2);
    pz = temp(:,3);
    apame_results_data.px=px;
    apame_results_data.py=py;
    apame_results_data.pz=pz;
  elseif line == ' P_STATIC'
    p_stat = read(results_file,panel_num_no_wake,1);
    apame_results_data.p_stat=p_stat;
  elseif line == ' P_DYNAMIC'
    p_dyna = read(results_file,panel_num_no_wake,1);
    apame_results_data.p_dyna=p_dyna;
  elseif line == ' P_MANOMETER'
    p_mano = read(results_file,panel_num_no_wake,1);
    apame_results_data.p_mano=p_mano;
  elseif line == ' end'
    ending = 1;
  end
end

file ('close',results_file)

endfunction
