// Copyright (C) 2011  Daniel Filkovic

// This file is part of APAME.

// APAME is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// APAME is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with APAME.  If not, see <http://www.gnu.org/licenses/>.

// file write_apame_input.sci

// This function writes ApameSolver input file based on given mesh
// and Apame data

function write_apame_input(file_name,nodes,elements,apame_data)

stacksize('max')
pi = 4.0*atan(1.0);

input_file=file('open',file_name,'unknown');

write(input_file,'APAME input file')
write(input_file,'VERSION 3.1')
write(input_file,('AIRSPEED '+string(apame_data.airspeed)))
write(input_file,('DENSITY '+string(apame_data.ro)))
write(input_file,('PRESSURE '+string(apame_data.p_ref)))
write(input_file,('MACH '+string(apame_data.mach)))
write(input_file,('CASE_NUM '+string(apame_data.case_num)))
for i = 1:apame_data.case_num
    write(input_file,apame_data.alfa(i)*180/pi,'(1x,E14.8,$)')
end
write(input_file,'')
for i = 1:apame_data.case_num
    write(input_file,apame_data.beta(i)*180/pi,'(1x,E14.8,$)')
end
write(input_file,'')
write(input_file,('WINGSPAN '+string(apame_data.wing_span)))
write(input_file,('MAC '+string(apame_data.mac)))
write(input_file,('SURFACE '+string(apame_data.wing_surf)))
write(input_file,'ORIGIN *')
write(input_file,[apame_data.origin(1), apame_data.origin(2),apame_data.origin(3)])
write(input_file,('METHOD '+string(apame_data.method)))
write(input_file,('ERROR '+string(apame_data.error)))
write(input_file,('COPL_ANG '+string(apame_data.coplanarity_angle)))
write(input_file,('COLLDIST '+string(apame_data.colldist)))
write(input_file,('FARFIELD '+string(apame_data.farfield)))
write(input_file,('COLLCALC '+string(apame_data.collcalc)))
write(input_file,('VELORDER '+string(apame_data.velorder)))
write(input_file,('RESULTS '+string(apame_data.res_req)))
write(input_file,('RES_COEF '+string(apame_data.res_coef)))
write(input_file,('RES_FORC '+string(apame_data.res_forc)))
write(input_file,('RES_GEOM '+string(apame_data.res_geom)))
write(input_file,('RES_VELO '+string(apame_data.res_velo)))
write(input_file,('RES_PRES '+string(apame_data.res_pres)))
write(input_file,('RES_CENT '+string(apame_data.res_cent)))
write(input_file,('RES_DOUB '+string(apame_data.res_doub)))
write(input_file,('RES_SORC '+string(apame_data.res_sorc)))
write(input_file,('RES_VELC '+string(apame_data.res_velc)))
write(input_file,('RES_MESH '+string(apame_data.res_mesh)))
write(input_file,('RES_STAT '+string(apame_data.res_stat)))
write(input_file,('RES_DYNA '+string(apame_data.res_dyna)))
write(input_file,('RES_MANO '+string(apame_data.res_mano)))

// write nodes
node_num=size(nodes,"r");
write(input_file,('NODES '+string(node_num)))
for i=1:node_num
    for j=1:3
        write(input_file,nodes(i,j),'(1x,E14.7,$)');
    end
    write(input_file,'')
end

// write elements
panel_num=size(elements,"r");
write(input_file,('PANELS '+string(panel_num)))
for i=1:panel_num
    for j=1:9
        write(input_file,elements(i,j),'(1x,I12,$)');
    end
    write(input_file,'')
end

write(input_file,'# end of input file','(a19)')

file('close',input_file);

endfunction
