// Copyright (C) 2010-2011  Daniel Filkovic

// This file is part of APAME.

// APAME is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// APAME is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with APAME.  If not, see <http://www.gnu.org/licenses/>.

// file ribs_dist.sci

// this function calculates y distribution of ribs based on global curvature

function [y_ribs,M] = ribs_dist(p_dih_der,p_dih_der2,p_swp_der2,p_crd_der2,p_aoa_der2,y_af,min_el,max_el,M_max,M_seg)

// calculating fine y distribution
yy_step = (y_af(M_seg)-y_af(1))/1000;    // first step
yy = y_af(1):yy_step:y_af(M_seg);        // then distribute linearly

// calculating second derivations on these y
curv_1 = polyval(p_dih_der2,yy);
curv_2 = polyval(p_swp_der2,yy);
curv_3 = polyval(p_crd_der2,yy);
curv_4 = polyval(p_aoa_der2,yy);

// getting global curvature
curv = sqrt(curv_1.^2 + curv_2.^2 + curv_3.^2 + curv_4.^2);

// modify curvature with denser mesh on boundaries
coef=2*max(curv(1),curv(length(curv)));
curv=curv + coef*((1-abs(sin(yy.*pi)))^3);

// getting linear dependence between element size and global curvature
p = polyfit([min(curv),max(curv)],[max_el,min_el],1);

// initalizing y_ribs
y_ribs = zeros(1,M_max);
i = 1;                  // counter of calculated ribs - y_ribs (unknown size)
j = 1;                  // index of vector yy (jumping)
while y_ribs(i) < y_af(M_seg)        // do until it gets to y end
    i = i+1;
    if i > M_max
        disp('ERROR: Maximum halfspan panel number reached, increase panel size!')
        break
    end
    el = polyval(p,curv(j));                // finding panel size depending on curvature
    y_ribs(i) = y_ribs(i-1) + el;           // next rib position
    j = round((y_ribs(i)-y_af(1))/yy_step); // find index for present curvature
end
M = i;
y_ribs = y_ribs(1:M);

// scaling y_ribs vector to nicely fit along wing span
coef = (y_ribs(i)-y_ribs(1)) / (y_af(M_seg)-y_af(1));    // calculating coefficient
y_ribs = (y_ribs-y_ribs(1))/coef + y_ribs(1);            // offset, scale, de-offset

endfunction
