// Copyright (C) 2010-2011  Daniel Filkovic

// This file is part of APAME.

// APAME is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// APAME is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with APAME.  If not, see <http://www.gnu.org/licenses/>.

// file rotat_seg.sci

// This function rotates airfoil segments by AOA in plane
// perpendicular to dihedral line and then rotates in x-axis to that plane

function [x_foil_new,y_foil_new,z_foil_new]=rotat_seg(x_foil,z_foil,aoa_af,p_dih_der,N,M_seg,y_af,wing_span)

halfspan = wing_span/2;

x_foil_new = zeros(N,M_seg);
y_foil_new = zeros(N,M_seg);
z_foil_new = zeros(N,M_seg);

// rotating airfoil data by AOA
for j = 1:M_seg
    cos_ang = cos(aoa_af(j));
    sin_ang = sin(aoa_af(j));
    for i = 1:N
        x_foil_new(i,j) = x_foil(i,j)*cos_ang + z_foil(i,j)*sin_ang;
        z_foil_new(i,j) = z_foil(i,j)*cos_ang - x_foil(i,j)*sin_ang;
    end
end

// rotating airfoil data by dihedral angle
for j = 1:M_seg
    ang = -atan(polyval(p_dih_der,y_af(j))/halfspan);
    for i = 1:N
        y_foil_new(i,j) = z_foil_new(i,j)*sin(ang);
        z_foil_new(i,j) = z_foil_new(i,j)*cos(ang);
    end
end

endfunction
