// Copyright (C) 2010-2011  Daniel Filkovic

// This file is part of APAME.

// APAME is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// APAME is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with APAME.  If not, see <http://www.gnu.org/licenses/>.

// file reader.sci

// This script is used for reading and displaying results from APAME
// solver. In order to run it, you must position to directory containing
// APAME results file (.res file) and edit line below with desired file.

clear
clc

// replace with APAME results file name
file_name = 'lofted_wing.res';
// enter 1 for smooth plot
smooth_plot = 1;
// select case number (for multiple cases)
plot_case = 1;
// enter 1 to plot geometry only (excluding wake panels)
body_only = 1;
// dummy color (types 20 and 21)
dummy_color=5;
// wake color (types 10 and 11)
wake_color=7;
// enable or disable field data cropping
crop=0;
// cropping boundaries
crop_min=-1.5;
crop_max=1;

//////////////////////////////////////////////////////////////////////////////////////

// open file
results_file = file('open',file_name,'old');

// read file
ending = 0;
while ending == 0
    line = read(results_file,1,1,"(A)");
    if line == ' PANEL_NUM'
        panel_num = read(results_file,1,1);
    elseif line == ' PANEL_NUM_NO_WAKE'
        panel_num_no_wake = read(results_file,1,1);
    elseif line == ' NODE_NUM'
        node_num = read(results_file,1,1);
    elseif line == ' NODES'
        temp = read(results_file,node_num,3);
        node_x = temp(:,1);
        node_y = temp(:,2);
        node_z = temp(:,3);
    elseif line == ' PANELS'
        temp =  read(results_file,panel_num,9);
        panel_type = temp(:,1);
        node_1 = temp(:,2);
        node_2 = temp(:,3);
        node_3 = temp(:,4);
        node_4 = temp(:,5);
        elem_1 = temp(:,6);
        elem_2 = temp(:,7);
        elem_3 = temp(:,8);
        elem_4 = temp(:,9);
    elseif line == ' AIRSPEED'
        airspeed = read(results_file,1,1);
    elseif line == ' DENSITY'
        density = read(results_file,1,1);
    elseif line == ' PRESSURE'
        pressure = read(results_file,1,1);
    elseif line == ' MACH'
        mach = read(results_file,1,1);
    elseif line == ' CASE_NUM'
        case_num = read(results_file,1,1);
        alfa = read(results_file,1,case_num);
        beta_ = read(results_file,1,case_num);
    elseif line == ' WINGSPAN'
        wingspan = read(results_file,1,1);
    elseif line == ' MAC'
        mac = read(results_file,1,1);
    elseif line == ' SURFACE'
        surface = read(results_file,1,1);
    elseif line == ' ORIGIN'
        origin = read(results_file,1,3);
    elseif line == ' METHOD'
        method = read(results_file,1,1);
    elseif line == ' ERROR'
        err = read(results_file,1,1);
    elseif line == ' COPL_ANG'
        copl_ang = read(results_file,1,1);
    elseif line == ' FARFIELD'
        farfield = read(results_file,1,1);
    elseif line == ' COLLCALC'
        collcalc = read(results_file,1,1);
    elseif line == ' VELORDER'
        velorder = read(results_file,1,1);
    elseif line == ' CX'
        CX = read(results_file,1,case_num);
    elseif line == ' CY'
        CY = read(results_file,1,case_num);
    elseif line == ' CZ'
        CZ = read(results_file,1,case_num);
    elseif line == ' CL'
        CL = read(results_file,1,case_num);
    elseif line == ' CM'
        CM = read(results_file,1,case_num);
    elseif line == ' CN'
        CN = read(results_file,1,case_num);
    elseif line == ' CDRAG'
        CDRAG = read(results_file,1,case_num);
    elseif line == ' CSIDE'
        CSIDE = read(results_file,1,case_num);
    elseif line == ' CLIFT'
        CLIFT = read(results_file,1,case_num);
    elseif line == ' FX'
        FX = read(results_file,1,case_num);
    elseif line == ' FY'
        FY = read(results_file,1,case_num);
    elseif line == ' FZ'
        FZ = read(results_file,1,case_num);
    elseif line == ' FL'
        FL = read(results_file,1,case_num);
    elseif line == ' FM'
        FM = read(results_file,1,case_num);
    elseif line == ' FN'
        FN = read(results_file,1,case_num);
    elseif line == ' FDRAG'
        FDRAG = read(results_file,1,case_num);
    elseif line == ' FSIDE'
        FSIDE = read(results_file,1,case_num);
    elseif line == ' FLIFT'
        FLIFT = read(results_file,1,case_num);
    elseif line == ' VELOCITY'
        v = read(results_file,panel_num_no_wake,case_num);
    elseif line == ' CP'
        cp = read(results_file,panel_num_no_wake,case_num);
    elseif line == ' CENTER POINTS'
        temp = read(results_file,panel_num,3);
        cx = temp(:,1);
        cy = temp(:,2);
        cz = temp(:,3);
    elseif line == ' DIPOLE'
        doublet = read(results_file,panel_num_no_wake,case_num);
    elseif line == ' SOURCE'
        source = read(results_file,panel_num_no_wake,case_num);
    elseif line == ' VELOCITY_COMP'
        for i=1:case_num
            temp = read(results_file,panel_num_no_wake,3);
            vx(:,i) = temp(:,1);
            vy(:,i) = temp(:,2);
            vz(:,i) = temp(:,3);
        end
    elseif line == ' S'
        S =  read(results_file,panel_num,1);
    elseif line == ' FF'
        FF = read(results_file,panel_num,1);
    elseif line == ' NORMAL'
        temp = read(results_file,panel_num,3);
        nx = temp(:,1);
        ny = temp(:,2);
        nz = temp(:,3);
    elseif line == ' L_VECTOR'
        temp = read(results_file,panel_num,3);
        lx = temp(:,1);
        ly = temp(:,2);
        lz = temp(:,3);
    elseif line == ' P_VECTOR'
        temp = read(results_file,panel_num,3);
        px = temp(:,1);
        py = temp(:,2);
        pz = temp(:,3);
    elseif line == ' P_STATIC'
        p_stat = read(results_file,panel_num_no_wake,1);
    elseif line == ' P_DYNAMIC'
        p_dyna = read(results_file,panel_num_no_wake,1);
    elseif line == ' P_MANOMETER'
        p_mano = read(results_file,panel_num_no_wake,1);
    elseif line == ' end'
        ending = 1;
    end
end

file ('close',results_file)

//////////////////////////////////////////////////////////////////////////////////////

// select which data to plot
field_data=cp;
if size(field_data,"c")==1 then
    field=field_data;
else
    field=field_data(:,plot_case);
end

// crop field data
if crop==1 then
    for i=1:length(field)
        if field(i) < crop_min
            field(i) = crop_min;
        elseif field(i) > crop_max
            field(i) = crop_max;
        end
    end
end

// scale and translate data to plot domain
field = field - min(field);
field = field/max(field)*511 + 1;

// calculate plot data in nodes
field_node = zeros(node_num,1);
node_sum = zeros(node_num,1);
for i=1:panel_num
    if panel_type(i) == 1
        field_node(node_1(i)) = field_node(node_1(i)) + field(i);
        field_node(node_2(i)) = field_node(node_2(i)) + field(i);
        field_node(node_3(i)) = field_node(node_3(i)) + field(i);
        field_node(node_4(i)) = field_node(node_4(i)) + field(i);
        node_sum(node_1(i)) = node_sum(node_1(i)) + 1;
        node_sum(node_2(i)) = node_sum(node_2(i)) + 1;
        node_sum(node_3(i)) = node_sum(node_3(i)) + 1;
        node_sum(node_4(i)) = node_sum(node_4(i)) + 1;
    elseif panel_type(i) == 2
        field_node(node_1(i)) = field_node(node_1(i)) + field(i);
        field_node(node_2(i)) = field_node(node_2(i)) + field(i);
        field_node(node_3(i)) = field_node(node_3(i)) + field(i);
        node_sum(node_1(i)) = node_sum(node_1(i)) + 1;
        node_sum(node_2(i)) = node_sum(node_2(i)) + 1;
        node_sum(node_3(i)) = node_sum(node_3(i)) + 1;
    end
end

// average the data
for i=1:node_num
    if node_sum(i) ~= 0
        field_node(i) = field_node(i) / node_sum(i);
    end
end

// count panel types
count_1=0;
count_2=0;
count_20=0;
count_21=0;
count_10=0;
count_11=0;
for i=1:panel_num
    if panel_type(i) == 1
        count_1 = count_1 + 1;
    elseif panel_type(i) == 2
        count_2 = count_2 + 1;
    elseif panel_type(i) == 20
        count_20 = count_20 + 1;
    elseif panel_type(i) == 21
        count_21 = count_21 + 1;
    elseif panel_type(i) == 10
        count_10 = count_10 + 1;
    elseif panel_type(i) == 11
        count_11 = count_11 + 1;
    end
end

// assemble plot data
j_1=0;
j_2=0;
j_10=0;
j_11=0;
j_20=0;
j_21=0;
X_1 = zeros(4,count_1);
Y_1 = zeros(4,count_1);
Z_1 = zeros(4,count_1);
X_2 = zeros(3,count_2);
Y_2 = zeros(3,count_2);
Z_2 = zeros(3,count_2);
X_20 = zeros(4,count_20);
Y_20 = zeros(4,count_20);
Z_20 = zeros(4,count_20);
X_21 = zeros(3,count_21);
Y_21 = zeros(3,count_21);
Z_21 = zeros(3,count_21);
X_10 = zeros(4,count_10);
Y_10 = zeros(4,count_10);
Z_10 = zeros(4,count_10);
X_11 = zeros(3,count_11);
Y_11 = zeros(3,count_11);
Z_11 = zeros(3,count_11);
if smooth_plot==1 then
    field_1 = zeros(4,count_1);
    field_2 = zeros(3,count_2);
else
    field_1 = zeros(count_1);
    field_2 = zeros(count_2);
end
for i=1:panel_num
    if panel_type(i) == 1
        j_1 = j_1 + 1;
        X_1(:,j_1) = [node_x(node_1(i));node_x(node_4(i));node_x(node_3(i));node_x(node_2(i))];
        Y_1(:,j_1) = [node_y(node_1(i));node_y(node_4(i));node_y(node_3(i));node_y(node_2(i))];
        Z_1(:,j_1) = [node_z(node_1(i));node_z(node_4(i));node_z(node_3(i));node_z(node_2(i))];
        if smooth_plot == 1
            field_1(:,j_1) = [field_node(node_1(i));field_node(node_4(i));field_node(node_3(i));field_node(node_2(i))];
        else
            field_1(j_1) = field(i);
        end
    elseif panel_type(i) == 2
        j_2 = j_2 + 1;
        X_2(:,j_2) = [node_x(node_1(i));node_x(node_3(i));node_x(node_2(i))];
        Y_2(:,j_2) = [node_y(node_1(i));node_y(node_3(i));node_y(node_2(i))];
        Z_2(:,j_2) = [node_z(node_1(i));node_z(node_3(i));node_z(node_2(i))];
        if smooth_plot == 1
            field_2(:,j_2) = [field_node(node_1(i));field_node(node_3(i));field_node(node_2(i))];
        else
            field_2(j_2) = field(i);
        end
    elseif panel_type(i) == 20
        j_20 = j_20 + 1;
        X_20(:,j_20) = [node_x(node_1(i));node_x(node_4(i));node_x(node_3(i));node_x(node_2(i))];
        Y_20(:,j_20) = [node_y(node_1(i));node_y(node_4(i));node_y(node_3(i));node_y(node_2(i))];
        Z_20(:,j_20) = [node_z(node_1(i));node_z(node_4(i));node_z(node_3(i));node_z(node_2(i))];
    elseif panel_type(i) == 21
        j_21 = j_21 + 1;
        X_21(:,j_21) = [node_x(node_1(i));node_x(node_4(i));node_x(node_3(i));node_x(node_2(i))];
        Y_21(:,j_21) = [node_y(node_1(i));node_y(node_4(i));node_y(node_3(i));node_y(node_2(i))];
        Z_21(:,j_21) = [node_z(node_1(i));node_z(node_4(i));node_z(node_3(i));node_z(node_2(i))];
    elseif panel_type(i) == 10
        j_10 = j_10 + 1;
        X_10(:,j_10) = [node_x(node_1(i));node_x(node_4(i));node_x(node_3(i));node_x(node_2(i))];
        Y_10(:,j_10) = [node_y(node_1(i));node_y(node_4(i));node_y(node_3(i));node_y(node_2(i))];
        Z_10(:,j_10) = [node_z(node_1(i));node_z(node_4(i));node_z(node_3(i));node_z(node_2(i))];
    elseif panel_type(i) == 11
        j_11 = j_11 + 1;
        X_11(:,j_11) = [node_x(node_1(i));node_x(node_4(i));node_x(node_3(i));node_x(node_2(i))];
        Y_11(:,j_11) = [node_y(node_1(i));node_y(node_4(i));node_y(node_3(i));node_y(node_2(i))];
        Z_11(:,j_11) = [node_z(node_1(i));node_z(node_4(i));node_z(node_3(i));node_z(node_2(i))];
    end
end

// plot data
if count_1 ~= 0
    plot3d(X_1,Y_1,list(Z_1,field_1))
    h = gce();
    h.color_mode=-2;    // remove boundary color
end
if count_2 ~= 0
    plot3d(X_2,Y_2,list(Z_2,field_2))
    h = gce();
    h.color_mode=-2;    // remove boundary color
end
if count_20 ~= 0
    plot3d(X_20,Y_20,Z_20)
    h = gce();
    h.color_mode=dummy_color;
end
if count_21 ~= 0
    plot3d(X_21,Y_21,Z_21)
    h = gce();
    h.color_mode=dummy_color;
end
if count_10 ~= 0 & body_only == 0
    plot3d(X_10,Y_10,Z_10)
    h = gce();
    h.color_mode=wake_color;
end
if count_11 ~= 0 & body_only == 0
    plot3d(X_11,Y_11,Z_11)
    h = gce();
    h.color_mode=wake_color;
end

// figure properties
f = gcf();
f.color_map = jetcolormap(512);
a = gca();
a.box = "off";
a.isoview = "on";
a.cube_scaling = "off";
a.axes_visible="on";
//////////////////////////////////////////////////////////////////////////////////////
