// Copyright (C) 2011  Daniel Filkovic

// This file is part of APAME.

// APAME is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// APAME is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with APAME.  If not, see <http://www.gnu.org/licenses/>.

// file root_tip.sci

// This function positions root and tip airfoil sections

function [xp,zp] = root_tip(x_foil,z_foil,wing_data,apame_data);

N=size(x_foil,"r");

xp = zeros(N,2);
zp = zeros(N,2);

// scaling root airfoil
xp(:,1) = x_foil(:,1)*wing_data.croot;
zp(:,1) = z_foil(:,1)*wing_data.croot;

// scaling tip airfoil
xptemp = x_foil(:,2)*wing_data.ctip;
zptemp = z_foil(:,2)*wing_data.ctip;

// tip airfoil rotation due to its angle of attack
if abs(wing_data.tipAL) < 1E-07
    xp(:,2) = xptemp;
    zp(:,2) = zptemp;
else
    for i = 1:N
        xp(i,2) =  xptemp(i)*cos(wing_data.tipAL)+zptemp(i)*sin(wing_data.tipAL);
        zp(i,2) = -xptemp(i)*sin(wing_data.tipAL)+zptemp(i)*cos(wing_data.tipAL);
    end
end

// translation of tip airfoil due to wing sweep
xp(:,2) = xp(:,2)+(apame_data.wing_span/2)*tan(wing_data.lambda14);

// translation of tip airfoil due dihedral
zp(:,2) = zp(:,2)+(apame_data.wing_span/2)*tan(wing_data.dihedral);

endfunction
