// Copyright (C) 2011  Daniel Filkovic

// This file is part of APAME.

// APAME is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// APAME is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with APAME.  If not, see <http://www.gnu.org/licenses/>.

// file simple_wing.sci

// This is the main calling function that creates wing mesh according to
// given arguments. Wing is trapezoidal, defined with root and tip
// airfoils, root and tip chords, dihedral angle and aerodynamic twist

function [nodes,elements,apame_data]=simple_wing(apame_data,wing_data,wake_rows,wake_length)

//==========================================================================
// AIRFOIL SEGMENTS CALCULATION
// reading airfoil coordinates from airfoil files
[x_foil,z_foil,N] = read_airfoils([wing_data.r_foil, wing_data.t_foil],wing_data.N_max,2);
// positioning root and tip airfoil
[xp,zp] = root_tip(x_foil,z_foil,wing_data,apame_data);
//__________________________________________________________________________
// MESHING WING
// interpolating panels on wing (full cosine distribution)
[x,y,z,N,M] = meshing(xp,zp,wing_data,apame_data);
//__________________________________________________________________________
// ADDITIONAL ACTIONS ON MESH
// convert structured mesh to unstructured (add mirror)
[apame_data,nodes,elements] = convert_to_unstructured(apame_data,N,M,x,y,z);
// calculate reference and wetted surface area and mean aerodynamic chord
[apame_data] = calculate_reference_data(apame_data,nodes,elements,N,M);
// create wake elements
[nodes,elements,apame_data] = create_wake(nodes,elements,N,M,x,y,z,wake_rows,wake_length,apame_data);
//__________________________________________________________________________

endfunction
